@@ -62,7 +62,7 @@ void *ucs2be_str(pool_t pool, const char *str, size_t *size);
  * Compute client -> server authentication response.
  */
 static void rpa_user_response(struct rpa_auth_request *request,
-			      unsigned char digest[MD5_RESULTLEN])
+			      unsigned char digest[STATIC_ARRAY MD5_RESULTLEN])
 {
 	struct md5_context ctx;
 	unsigned char z[48];
@@ -86,7 +86,7 @@ static void rpa_user_response(struct rpa_auth_request *request,
  * Compute server -> client authentication response.
  */
 static void rpa_server_response(struct rpa_auth_request *request,
-				unsigned char digest[MD5_RESULTLEN])
+				unsigned char digest[STATIC_ARRAY MD5_RESULTLEN])
 {
 	struct md5_context ctx;
 	unsigned char tmp[MD5_RESULTLEN];
