@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2015 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2007-2016 Dovecot authors, see the included COPYING file */
 
 /* Currently supports only GLIBC-compatible NSS modules */
 
@@ -127,7 +127,7 @@ userdb_nss_preinit(pool_t pool, const char *args)
 		i_fatal("userdb nss: Missing service");
 	userdb_nss_load_module(module, pool);
 
-	module->module.cache_key = USER_CACHE_KEY;
+	module->module.default_cache_key = USER_CACHE_KEY;
 	return &module->module;
 }
 
