@@ -13,6 +13,8 @@
 #include "dict-commands.h"
 #include "main.h"
 
+#define DICT_CLIENT_PROTOCOL_TIMINGS_MIN_VERSION 1
+#define DICT_CLIENT_PROTOCOL_UNORDERED_MIN_VERSION 1
 #define DICT_OUTPUT_OPTIMAL_SIZE 1024
 
 struct dict_cmd_func {
@@ -29,7 +31,8 @@ struct dict_connection_cmd {
 	struct dict_iterate_context *iter;
 	enum dict_iterate_flags iter_flags;
 
-	unsigned int trans_id;
+	unsigned int async_reply_id;
+	unsigned int trans_id; /* obsolete */
 };
 
 struct dict_command_stats cmd_stats;
@@ -69,11 +72,15 @@ static void dict_connection_cmds_flush(struct dict_connection *conn)
 {
 	struct dict_connection_cmd *cmd, *const *first_cmdp;
 
+	i_assert(conn->minor_version < DICT_CLIENT_PROTOCOL_UNORDERED_MIN_VERSION);
+
 	dict_connection_ref(conn);
 	while (array_count(&conn->cmds) > 0) {
 		first_cmdp = array_idx(&conn->cmds, 0);
 		cmd = *first_cmdp;
 
+		i_assert(cmd->async_reply_id == 0);
+
 		/* we may be able to start outputting iterations now. */
 		if (cmd->iter != NULL)
 			(void)cmd_iterate_flush(cmd);
@@ -89,6 +96,37 @@ static void dict_connection_cmds_flush(struct dict_connection *conn)
 	dict_connection_unref_safe(conn);
 }
 
+static void dict_connection_cmd_try_flush(struct dict_connection_cmd **_cmd)
+{
+	struct dict_connection_cmd *cmd = *_cmd;
+
+	*_cmd = NULL;
+	if (cmd->conn->minor_version < DICT_CLIENT_PROTOCOL_UNORDERED_MIN_VERSION) {
+		dict_connection_cmds_flush(cmd->conn);
+		return;
+	}
+	i_assert(cmd->async_reply_id != 0);
+	i_assert(cmd->reply != NULL);
+
+	o_stream_nsend_str(cmd->conn->output, t_strdup_printf("%c%u\t%s",
+		DICT_PROTOCOL_REPLY_ASYNC_REPLY,
+		cmd->async_reply_id, cmd->reply));
+	dict_connection_cmd_remove(cmd);
+}
+
+static void dict_connection_cmd_async(struct dict_connection_cmd *cmd)
+{
+	if (cmd->conn->minor_version < DICT_CLIENT_PROTOCOL_UNORDERED_MIN_VERSION)
+		return;
+
+	i_assert(cmd->async_reply_id == 0);
+	cmd->async_reply_id = ++cmd->conn->async_id_counter;
+	if (cmd->async_reply_id == 0)
+		cmd->async_reply_id = ++cmd->conn->async_id_counter;
+	o_stream_nsend_str(cmd->conn->output, t_strdup_printf("%c%u\n",
+		DICT_PROTOCOL_REPLY_ASYNC_ID, cmd->async_reply_id));
+}
+
 static void
 cmd_stats_update(struct dict_connection_cmd *cmd, struct timing *timing)
 {
@@ -97,7 +135,6 @@ cmd_stats_update(struct dict_connection_cmd *cmd, struct timing *timing)
 	if (!dict_settings->verbose_proctitle)
 		return;
 
-	io_loop_time_refresh();
 	diff = timeval_diff_usecs(&ioloop_timeval, &cmd->start_timeval);
 	if (diff < 0)
 		diff = 0;
@@ -105,41 +142,83 @@ cmd_stats_update(struct dict_connection_cmd *cmd, struct timing *timing)
 	dict_proctitle_update_later();
 }
 
+static void
+dict_cmd_reply_handle_timings(struct dict_connection_cmd *cmd,
+			      string_t *str, struct timing *timing)
+{
+	io_loop_time_refresh();
+	cmd_stats_update(cmd, timing);
+
+	if (cmd->conn->minor_version < DICT_CLIENT_PROTOCOL_TIMINGS_MIN_VERSION)
+		return;
+	str_printfa(str, "\t%ld\t%u\t%ld\t%u",
+		    (long)cmd->start_timeval.tv_sec,
+		    (unsigned int)cmd->start_timeval.tv_usec,
+		    (long)ioloop_timeval.tv_sec,
+		    (unsigned int)ioloop_timeval.tv_usec);
+}
+
 static void
 cmd_lookup_callback(const struct dict_lookup_result *result, void *context)
 {
 	struct dict_connection_cmd *cmd = context;
-
-	cmd_stats_update(cmd, cmd_stats.lookups);
+	string_t *str = t_str_new(128);
 
 	if (result->ret > 0) {
-		cmd->reply = i_strdup_printf("%c%s\n",
-			DICT_PROTOCOL_REPLY_OK, str_tabescape(result->value));
+		str_append_c(str, DICT_PROTOCOL_REPLY_OK);
+		str_append_tabescaped(str, result->value);
 	} else if (result->ret == 0) {
-		cmd->reply = i_strdup_printf("%c\n", DICT_PROTOCOL_REPLY_NOTFOUND);
+		str_append_c(str, DICT_PROTOCOL_REPLY_NOTFOUND);
 	} else {
 		i_error("%s", result->error);
-		cmd->reply = i_strdup_printf("%c\n", DICT_PROTOCOL_REPLY_FAIL);
+		str_append_c(str, DICT_PROTOCOL_REPLY_FAIL);
+		str_append_tabescaped(str, result->error);
 	}
-	dict_connection_cmds_flush(cmd->conn);
+	dict_cmd_reply_handle_timings(cmd, str, cmd_stats.lookups);
+	str_append_c(str, '\n');
+
+	cmd->reply = i_strdup(str_c(str));
+	dict_connection_cmd_try_flush(&cmd);
 }
 
 static int cmd_lookup(struct dict_connection_cmd *cmd, const char *line)
 {
 	/* <key> */
+	dict_connection_cmd_async(cmd);
 	dict_lookup_async(cmd->conn->dict, line, cmd_lookup_callback, cmd);
 	return 1;
 }
 
+static bool dict_connection_flush_if_full(struct dict_connection *conn)
+{
+	if (o_stream_get_buffer_used_size(conn->output) >
+	    DICT_OUTPUT_OPTIMAL_SIZE) {
+		if (o_stream_flush(conn->output) <= 0) {
+			/* continue later when there's more space
+			   in output buffer */
+			o_stream_set_flush_pending(conn->output, TRUE);
+			return FALSE;
+		}
+		/* flushed everything, continue */
+	}
+	return TRUE;
+}
+
 static int cmd_iterate_flush(struct dict_connection_cmd *cmd)
 {
 	string_t *str;
 	const char *key, *value;
 
+	if (!dict_connection_flush_if_full(cmd->conn))
+		return 0;
+
 	str = t_str_new(256);
-	o_stream_cork(cmd->conn->output);
 	while (dict_iterate(cmd->iter, &key, &value)) {
 		str_truncate(str, 0);
+		if (cmd->async_reply_id != 0) {
+			str_append_c(str, DICT_PROTOCOL_REPLY_ASYNC_REPLY);
+			str_printfa(str, "%u\t", cmd->async_reply_id);
+		}
 		str_append_c(str, DICT_PROTOCOL_REPLY_OK);
 		str_append_tabescaped(str, key);
 		str_append_c(str, '\t');
@@ -148,17 +227,8 @@ static int cmd_iterate_flush(struct dict_connection_cmd *cmd)
 		str_append_c(str, '\n');
 		o_stream_nsend(cmd->conn->output, str_data(str), str_len(str));
 
-		if (o_stream_get_buffer_used_size(cmd->conn->output) >
-		    DICT_OUTPUT_OPTIMAL_SIZE) {
-			if (o_stream_flush(cmd->conn->output) <= 0) {
-				/* continue later when there's more space
-				   in output buffer */
-				o_stream_uncork(cmd->conn->output);
-				o_stream_set_flush_pending(cmd->conn->output, TRUE);
-				return 0;
-			}
-			/* flushed everything, continue */
-		}
+		if (!dict_connection_flush_if_full(cmd->conn))
+			return 0;
 	}
 	if (dict_iterate_has_more(cmd->iter)) {
 		/* wait for the next iteration callback */
@@ -168,10 +238,9 @@ static int cmd_iterate_flush(struct dict_connection_cmd *cmd)
 	str_truncate(str, 0);
 	if (dict_iterate_deinit(&cmd->iter) < 0)
 		str_append_c(str, DICT_PROTOCOL_REPLY_FAIL);
+	dict_cmd_reply_handle_timings(cmd, str, cmd_stats.iterations);
 	str_append_c(str, '\n');
-	o_stream_uncork(cmd->conn->output);
 
-	cmd_stats_update(cmd, cmd_stats.iterations);
 	cmd->reply = i_strdup(str_c(str));
 	return 1;
 }
@@ -179,26 +248,36 @@ static int cmd_iterate_flush(struct dict_connection_cmd *cmd)
 static void cmd_iterate_callback(void *context)
 {
 	struct dict_connection_cmd *cmd = context;
+	struct dict_connection *conn = cmd->conn;
 
+	dict_connection_ref(conn);
+	o_stream_cork(conn->output);
 	dict_connection_cmd_output_more(cmd);
+	o_stream_uncork(conn->output);
+	dict_connection_unref_safe(conn);
 }
 
 static int cmd_iterate(struct dict_connection_cmd *cmd, const char *line)
 {
 	const char *const *args;
 	unsigned int flags;
+	uint64_t max_rows;
 
 	args = t_strsplit_tabescaped(line);
-	if (str_array_length(args) < 2 ||
-	    str_to_uint(args[0], &flags) < 0) {
+	if (str_array_length(args) < 3 ||
+	    str_to_uint(args[0], &flags) < 0 ||
+	    str_to_uint64(args[1], &max_rows) < 0) {
 		i_error("dict client: ITERATE: broken input");
 		return -1;
 	}
+	dict_connection_cmd_async(cmd);
 
-	/* <flags> <path> */
+	/* <flags> <max_rows> <path> */
 	flags |= DICT_ITERATE_FLAG_ASYNC;
-	cmd->iter = dict_iterate_init_multiple(cmd->conn->dict, args+1, flags);
+	cmd->iter = dict_iterate_init_multiple(cmd->conn->dict, args+2, flags);
 	cmd->iter_flags = flags;
+	if (max_rows > 0)
+		dict_iterate_set_limit(cmd->iter, max_rows);
 	dict_iterate_set_async_callback(cmd->iter, cmd_iterate_callback, cmd);
 	dict_connection_cmd_output_more(cmd);
 	return 1;
@@ -285,10 +364,9 @@ dict_connection_transaction_lookup_parse(struct dict_connection *conn,
 static void
 cmd_commit_finish(struct dict_connection_cmd *cmd, int ret, bool async)
 {
+	string_t *str = t_str_new(64);
 	char chr;
 
-	cmd_stats_update(cmd, cmd_stats.commits);
-
 	switch (ret) {
 	case 1:
 		chr = DICT_PROTOCOL_REPLY_OK;
@@ -305,13 +383,17 @@ cmd_commit_finish(struct dict_connection_cmd *cmd, int ret, bool async)
 		break;
 	}
 	if (async) {
-		cmd->reply = i_strdup_printf("%c%c%u\n",
+		str_printfa(str, "%c%c%u",
 			DICT_PROTOCOL_REPLY_ASYNC_COMMIT, chr, cmd->trans_id);
 	} else {
-		cmd->reply = i_strdup_printf("%c%u\n", chr, cmd->trans_id);
+		str_printfa(str, "%c%u", chr, cmd->trans_id);
 	}
+	dict_cmd_reply_handle_timings(cmd, str, cmd_stats.commits);
+	str_append_c(str, '\n');
+	cmd->reply = i_strdup(str_c(str));
+
 	dict_connection_transaction_array_remove(cmd->conn, cmd->trans_id);
-	dict_connection_cmds_flush(cmd->conn);
+	dict_connection_cmd_try_flush(&cmd);
 }
 
 static void cmd_commit_callback(int ret, void *context)
@@ -337,6 +419,7 @@ cmd_commit(struct dict_connection_cmd *cmd, const char *line)
 		return -1;
 	cmd->trans_id = trans->id;
 
+	dict_connection_cmd_async(cmd);
 	dict_transaction_commit_async(&trans->ctx, cmd_commit_callback, cmd);
 	return 1;
 }
@@ -350,6 +433,7 @@ cmd_commit_async(struct dict_connection_cmd *cmd, const char *line)
 		return -1;
 	cmd->trans_id = trans->id;
 
+	dict_connection_cmd_async(cmd);
 	dict_transaction_commit_async(&trans->ctx, cmd_commit_callback_async, cmd);
 	return 1;
 }
@@ -493,38 +577,50 @@ int dict_command_input(struct dict_connection *conn, const char *line)
 	return 0;
 }
 
-static void dict_connection_cmd_output_more(struct dict_connection_cmd *cmd)
+static bool dict_connection_cmds_try_output_more(struct dict_connection *conn)
 {
-	struct dict_connection_cmd *const *first_cmdp;
-	
-	first_cmdp = array_idx(&cmd->conn->cmds, 0);
-	if (*first_cmdp == cmd) {
-		if (cmd_iterate_flush(cmd) > 0)
-			dict_connection_cmds_flush(cmd->conn);
+	struct dict_connection_cmd *const *cmdp, *cmd;
+
+	/* only iterators may be returning a lot of data */
+	array_foreach(&conn->cmds, cmdp) {
+		cmd = *cmdp;
+
+		if (cmd->iter == NULL) {
+			/* not an iterator */
+		} else if (cmd_iterate_flush(cmd) == 0) {
+			/* unfinished */
+		} else {
+			dict_connection_cmd_try_flush(&cmd);
+			/* cmd should be freed now, restart output */
+			return TRUE;
+		}
+		if (conn->minor_version < DICT_CLIENT_PROTOCOL_TIMINGS_MIN_VERSION)
+			break;
+		/* try to flush the rest */
 	}
+	return FALSE;
 }
 
 void dict_connection_cmds_output_more(struct dict_connection *conn)
 {
-	struct dict_connection_cmd *cmd, *const *first_cmdp;
-
-	/* only iterators may be returning a lot of data */
 	while (array_count(&conn->cmds) > 0) {
-		first_cmdp = array_idx(&conn->cmds, 0);
-		cmd = *first_cmdp;
-
-		if (cmd->iter == NULL)
+		if (!dict_connection_cmds_try_output_more(conn))
 			break;
-
-		if (cmd_iterate_flush(cmd) == 0) {
-			/* unfinished */
-			break;
-		}
-		dict_connection_cmds_flush(cmd->conn);
-		/* cmd should be freed now */
 	}
 }
 
+static void dict_connection_cmd_output_more(struct dict_connection_cmd *cmd)
+{
+	struct dict_connection_cmd *const *first_cmdp;
+
+	if (cmd->conn->minor_version < DICT_CLIENT_PROTOCOL_TIMINGS_MIN_VERSION) {
+		first_cmdp = array_idx(&cmd->conn->cmds, 0);
+		if (*first_cmdp != cmd)
+			return;
+	}
+	(void)dict_connection_cmds_try_output_more(cmd->conn);
+}
+
 void dict_commands_init(void)
 {
 	cmd_stats.lookups = timing_init();
