@@ -2,8 +2,12 @@
 
 #include "lib.h"
 #include "restrict-access.h"
+#include "ioloop.h"
 #include "randgen.h"
+#include "str.h"
 #include "hostpid.h"
+#include "timing.h"
+#include "process-title.h"
 #include "env-util.h"
 #include "module-dir.h"
 #include "master-service.h"
@@ -11,10 +15,54 @@
 #include "sql-api.h"
 #include "dict.h"
 #include "dict-client.h"
+#include "dict-commands.h"
 #include "dict-connection.h"
 #include "dict-settings.h"
+#include "main.h"
 
 static struct module *modules;
+static struct timeout *to_proctitle;
+static bool proctitle_updated;
+
+static void
+add_timing_string(string_t *str, struct timing *timing, const char *name)
+{
+	str_printfa(str, ", %u %s:%llu/%llu/%llu/%llu",
+		    timing_get_count(timing), name,
+		    (unsigned long long)timing_get_min(timing)/1000,
+		    (unsigned long long)timing_get_avg(timing)/1000,
+		    (unsigned long long)timing_get_95th(timing)/1000,
+		    (unsigned long long)timing_get_max(timing)/1000);
+	timing_reset(timing);
+}
+
+static void dict_proctitle_update(void *context ATTR_UNUSED)
+{
+	string_t *str = t_str_new(128);
+
+	if (!proctitle_updated)
+		timeout_remove(&to_proctitle);
+
+	str_printfa(str, "[%u clients", dict_connections_current_count());
+
+	add_timing_string(str, cmd_stats.lookups, "lookups");
+	add_timing_string(str, cmd_stats.iterations, "iters");
+	add_timing_string(str, cmd_stats.commits, "commits");
+	str_append_c(str, ']');
+
+	process_title_set(str_c(str));
+	proctitle_updated = FALSE;
+}
+
+void dict_proctitle_update_later(void)
+{
+	if (!dict_settings->verbose_proctitle)
+		return;
+
+	if (to_proctitle == NULL)
+		to_proctitle = timeout_add(1000, dict_proctitle_update, NULL);
+	proctitle_updated = TRUE;
+}
 
 static void dict_die(void)
 {
@@ -68,12 +116,17 @@ static void main_init(void)
 	/* Register only after loading modules. They may contain SQL drivers,
 	   which we'll need to register. */
 	dict_drivers_register_all();
+	dict_commands_init();
 }
 
 static void main_deinit(void)
 {
+	if (to_proctitle != NULL)
+		timeout_remove(&to_proctitle);
+
 	dict_connections_destroy_all();
 	dict_drivers_unregister_all();
+	dict_commands_deinit();
 
 	module_dir_unload(&modules);
 
@@ -83,8 +136,7 @@ static void main_deinit(void)
 
 int main(int argc, char *argv[])
 {
-	const enum master_service_flags service_flags =
-		MASTER_SERVICE_FLAG_UPDATE_PROCTITLE;
+	const enum master_service_flags service_flags = 0;
 	const struct setting_parser_info *set_roots[] = {
 		&dict_setting_parser_info,
 		NULL
