@@ -5,8 +5,8 @@ struct director;
 struct director_request;
 
 typedef void
-director_request_callback(const struct ip_addr *ip, const char *errormsg,
-			  void *context);
+director_request_callback(const struct ip_addr *ip, const char *hostname,
+			  const char *errormsg, void *context);
 
 void director_request(struct director *dir, const char *username,
 		      const char *tag,
