@@ -12,6 +12,7 @@
 #include "istream.h"
 #include "ostream.h"
 #include "iostream-temp.h"
+#include "mail-user-hash.h"
 #include "user-directory.h"
 #include "mail-host.h"
 #include "director-host.h"
@@ -628,6 +629,8 @@ void director_remove_host(struct director *dir, struct director_host *src,
 			  struct director_host *orig_src,
 			  struct mail_host *host)
 {
+	struct user_directory *users = host->tag->users;
+
 	if (src != NULL) {
 		if (orig_src == NULL) {
 			orig_src = dir->self_host;
@@ -640,7 +643,7 @@ void director_remove_host(struct director *dir, struct director_host *src,
 			orig_src->last_seq, net_ip2addr(&host->ip)));
 	}
 
-	user_directory_remove_host(dir->users, host);
+	user_directory_remove_host(users, host);
 	mail_host_remove(host);
 	director_sync(dir);
 }
@@ -649,6 +652,8 @@ void director_flush_host(struct director *dir, struct director_host *src,
 			 struct director_host *orig_src,
 			 struct mail_host *host)
 {
+	struct user_directory *users = host->tag->users;
+
 	if (orig_src == NULL) {
 		orig_src = dir->self_host;
 		orig_src->last_seq++;
@@ -658,7 +663,7 @@ void director_flush_host(struct director *dir, struct director_host *src,
 		"HOST-FLUSH\t%s\t%u\t%u\t%s\n",
 		net_ip2addr(&orig_src->ip), orig_src->port, orig_src->last_seq,
 		net_ip2addr(&host->ip)));
-	user_directory_remove_host(dir->users, host);
+	user_directory_remove_host(users, host);
 	director_sync(dir);
 }
 
@@ -714,11 +719,13 @@ static void
 director_flush_user_continue(int result, struct director_kill_context *ctx)
 {
 	struct director *dir = ctx->dir;
-	struct user *user =
-		user_directory_lookup(dir->users, ctx->username_hash);
-
 	ctx->callback_pending = FALSE;
 
+	struct user *user = user_directory_lookup(ctx->tag->users,
+						  ctx->username_hash);
+	if (user != NULL)
+		director_user_kill_finish_delayed(dir, user,result == 1);
+
 	if (result == 0) {
 		struct istream *is = iostream_temp_finish(&ctx->reply, (size_t)-1);
 		char *data;
@@ -935,7 +942,7 @@ static void director_kill_user_callback(enum ipc_client_cmd_state state,
 
 	ctx->callback_pending = FALSE;
 
-	user = user_directory_lookup(ctx->dir->users, ctx->username_hash);
+	user = user_directory_lookup(ctx->tag->users, ctx->username_hash);
 	if (!DIRECTOR_KILL_CONTEXT_IS_VALID(user, ctx)) {
 		/* user was already freed - ignore */
 		i_assert(ctx->to_move == NULL);
@@ -972,7 +979,8 @@ static void director_user_move_timeout(struct user *user)
 
 static void
 director_kill_user(struct director *dir, struct director_host *src,
-		   struct user *user, struct mail_host *old_host)
+		   struct user *user, struct mail_tag *tag,
+		   struct mail_host *old_host)
 {
 	struct director_kill_context *ctx;
 	const char *cmd;
@@ -989,6 +997,7 @@ director_kill_user(struct director *dir, struct director_host *src,
 
 	user->kill_ctx = ctx = i_new(struct director_kill_context, 1);
 	ctx->dir = dir;
+	ctx->tag = tag;
 	ctx->username_hash = user->username_hash;
 	ctx->kill_is_self_initiated = src->self;
 	if (old_host != NULL)
@@ -1018,6 +1027,7 @@ void director_move_user(struct director *dir, struct director_host *src,
 			struct director_host *orig_src,
 			unsigned int username_hash, struct mail_host *host)
 {
+	struct user_directory *users = host->tag->users;
 	struct user *user;
 
 	/* 1. move this user's host, and set its "killing" flag to delay all of
@@ -1036,11 +1046,11 @@ void director_move_user(struct director *dir, struct director_host *src,
 	   5. after receiving USER-KILLED-EVERYWHERE notification,
 	   new connections are again allowed for the user.
 	*/
-	user = user_directory_lookup(dir->users, username_hash);
+	user = user_directory_lookup(users, username_hash);
 	if (user == NULL) {
-		user = user_directory_add(dir->users, username_hash,
+		user = user_directory_add(users, username_hash,
 					  host, ioloop_time);
-		director_kill_user(dir, src, user, NULL);
+		director_kill_user(dir, src, user, host->tag, NULL);
 	} else {
 		struct mail_host *old_host = user->host;
 
@@ -1052,7 +1062,7 @@ void director_move_user(struct director *dir, struct director_host *src,
 		user->host = host;
 		user->host->user_count++;
 		user->timestamp = ioloop_time;
-		director_kill_user(dir, src, user, old_host);
+		director_kill_user(dir, src, user, old_host->tag, old_host);
 	}
 
 	if (orig_src == NULL) {
@@ -1157,11 +1167,13 @@ director_send_user_killed_everywhere(struct director *dir,
 		username_hash));
 }
 
-void director_user_killed(struct director *dir, unsigned int username_hash)
+static void
+director_user_tag_killed(struct director *dir, struct mail_tag *tag,
+			 unsigned int username_hash)
 {
 	struct user *user;
 
-	user = user_directory_lookup(dir->users, username_hash);
+	user = user_directory_lookup(tag->users, username_hash);
 	if (user == NULL || !USER_IS_BEING_KILLED(user))
 		return;
 
@@ -1192,14 +1204,24 @@ void director_user_killed(struct director *dir, unsigned int username_hash)
 	}
 }
 
-void director_user_killed_everywhere(struct director *dir,
-				     struct director_host *src,
-				     struct director_host *orig_src,
-				     unsigned int username_hash)
+void director_user_killed(struct director *dir, unsigned int username_hash)
+{
+	struct mail_tag *const *tagp;
+
+	array_foreach(mail_hosts_get_tags(dir->mail_hosts), tagp)
+		director_user_tag_killed(dir, *tagp, username_hash);
+}
+
+static void
+director_user_tag_killed_everywhere(struct director *dir,
+				    struct mail_tag *tag,
+				    struct director_host *src,
+				    struct director_host *orig_src,
+				    unsigned int username_hash)
 {
 	struct user *user;
 
-	user = user_directory_lookup(dir->users, username_hash);
+	user = user_directory_lookup(tag->users, username_hash);
 	if (user == NULL) {
 		dir_debug("User %u no longer exists - ignoring USER-KILLED-EVERYWHERE",
 			  username_hash);
@@ -1220,6 +1242,19 @@ void director_user_killed_everywhere(struct director *dir,
 	director_send_user_killed_everywhere(dir, src, orig_src, username_hash);
 }
 
+void director_user_killed_everywhere(struct director *dir,
+				     struct director_host *src,
+				     struct director_host *orig_src,
+				     unsigned int username_hash)
+{
+	struct mail_tag *const *tagp;
+
+	array_foreach(mail_hosts_get_tags(dir->mail_hosts), tagp) {
+		director_user_tag_killed_everywhere(dir, *tagp, src, orig_src,
+						    username_hash);
+	}
+}
+
 static void director_state_callback_timeout(struct director *dir)
 {
 	timeout_remove(&dir->to_callback);
@@ -1288,10 +1323,9 @@ director_init(const struct director_settings *set,
 	i_array_init(&dir->dir_hosts, 16);
 	i_array_init(&dir->pending_requests, 16);
 	i_array_init(&dir->connections, 8);
-	dir->users = user_directory_init(set->director_user_expire,
-					 set->director_username_hash,
-					 director_user_freed);
-	dir->mail_hosts = mail_hosts_init(set->director_consistent_hashing);
+	dir->mail_hosts = mail_hosts_init(set->director_user_expire,
+					  set->director_consistent_hashing,
+					  director_user_freed);
 
 	dir->ipc_proxy = ipc_client_init(DIRECTOR_IPC_PROXY_PATH);
 	dir->ring_min_version = DIRECTOR_VERSION_MINOR;
@@ -1312,7 +1346,6 @@ void director_deinit(struct director **_dir)
 		director_connection_deinit(&conn, "Shutting down");
 	}
 
-	user_directory_deinit(&dir->users);
 	mail_hosts_deinit(&dir->mail_hosts);
 	mail_hosts_deinit(&dir->orig_config_hosts);
 
@@ -1354,6 +1387,58 @@ void dir_debug(const char *fmt, ...)
 	va_end(args);
 }
 
+struct director_user_iter {
+	struct director *dir;
+	unsigned int tag_idx;
+	struct user_directory_iter *user_iter;
+};
+
+struct director_user_iter *director_iterate_users_init(struct director *dir)
+{
+	struct director_user_iter *iter = i_new(struct director_user_iter, 1);
+	iter->dir = dir;
+	return iter;
+}
+
+struct user *director_iterate_users_next(struct director_user_iter *iter)
+{
+	const ARRAY_TYPE(mail_tag) *tags;
+	struct user *user;
+
+	i_assert(iter != NULL);
+
+	if (iter->user_iter == NULL) {
+		tags = mail_hosts_get_tags(iter->dir->mail_hosts);
+		if (iter->tag_idx >= array_count(tags))
+			return NULL;
+		struct mail_tag *const *tagp = array_idx(tags, iter->tag_idx);
+		iter->user_iter = user_directory_iter_init((*tagp)->users);
+	}
+	user = user_directory_iter_next(iter->user_iter);
+	if (user == NULL) {
+		user_directory_iter_deinit(&iter->user_iter);
+		iter->tag_idx++;
+		return director_iterate_users_next(iter);
+	} else
+		return user;
+}
+
+void director_iterate_users_deinit(struct director_user_iter **_iter)
+{
+	i_assert(_iter != NULL && *_iter != NULL);
+	struct director_user_iter *iter = *_iter;
+	*_iter = NULL;
+	if (iter->user_iter != NULL)
+		user_directory_iter_deinit(&iter->user_iter);
+	i_free(iter);
+}
+
+unsigned int
+director_get_username_hash(struct director *dir, const char *username)
+{
+	return mail_user_hash(username, dir->set->director_username_hash);
+}
+
 void directors_init(void)
 {
 	user_move_throttle =
