@@ -31,7 +31,7 @@ struct director_reset_cmd {
 
 	struct director *dir;
 	struct doveadm_connection *_conn;
-	struct user_directory_iter *iter;
+	struct director_user_iter *iter;
 	unsigned int host_idx, hosts_count;
 	unsigned int max_moving_users;
 };
@@ -82,7 +82,9 @@ static void doveadm_cmd_host_list_removed(struct doveadm_connection *conn)
 	string_t *str = t_str_new(1024);
 	int ret;
 
-	orig_hosts_list = mail_hosts_init(conn->dir->set->director_consistent_hashing);
+	orig_hosts_list = mail_hosts_init(conn->dir->set->director_user_expire,
+					  conn->dir->set->director_consistent_hashing,
+					  NULL);
 	(void)mail_hosts_parse_and_add(orig_hosts_list,
 				       conn->dir->set->director_mail_servers);
 
@@ -428,7 +430,7 @@ static void doveadm_reset_cmd_free(struct director_reset_cmd *cmd)
 	DLLIST_REMOVE(&reset_cmds, cmd);
 
 	if (cmd->iter != NULL)
-		user_directory_iter_deinit(&cmd->iter);
+		director_iterate_users_deinit(&cmd->iter);
 	if (cmd->_conn != NULL)
 		cmd->_conn->reset_cmd = NULL;
 	i_free(cmd);
@@ -449,8 +451,9 @@ director_host_reset_users(struct director_reset_cmd *cmd,
 		director_connection_cork(dir->right);
 
 	if (cmd->iter == NULL)
-		cmd->iter = user_directory_iter_init(dir->users);
-	while ((user = user_directory_iter_next(cmd->iter)) != NULL) {
+		cmd->iter = director_iterate_users_init(dir);
+
+	while ((user = director_iterate_users_next(cmd->iter)) != NULL) {
 		if (user->host != host)
 			continue;
 		new_host = mail_host_get_by_hash(dir->mail_hosts,
@@ -464,7 +467,7 @@ director_host_reset_users(struct director_reset_cmd *cmd,
 			break;
 	}
 	if (user == NULL)
-		user_directory_iter_deinit(&cmd->iter);
+		director_iterate_users_deinit(&cmd->iter);
 	if (dir->right != NULL)
 		director_connection_uncork(dir->right);
 	return user == NULL;
@@ -555,6 +558,7 @@ doveadm_cmd_user_lookup(struct doveadm_connection *conn,
 	struct mail_host *host;
 	const char *username, *tag;
 	unsigned int username_hash;
+	struct mail_tag *mail_tag;
 	string_t *str = t_str_new(256);
 
 	if (args[0] == NULL) {
@@ -565,10 +569,12 @@ doveadm_cmd_user_lookup(struct doveadm_connection *conn,
 		tag = args[1] != NULL ? args[1] : "";
 	}
 	if (str_to_uint(username, &username_hash) < 0)
-		username_hash = user_directory_get_username_hash(conn->dir->users, username);
+		username_hash = director_get_username_hash(conn->dir, username);
 
 	/* get user's current host */
-	user = user_directory_lookup(conn->dir->users, username_hash);
+	mail_tag = mail_tag_find(conn->dir->mail_hosts, tag);
+	user = mail_tag == NULL ? NULL :
+		user_directory_lookup(mail_tag->users, username_hash);
 	if (user == NULL)
 		str_append(str, "\t0");
 	else {
@@ -598,7 +604,7 @@ doveadm_cmd_user_lookup(struct doveadm_connection *conn,
 static int
 doveadm_cmd_user_list(struct doveadm_connection *conn, const char *const *args)
 {
-	struct user_directory_iter *iter;
+	struct director_user_iter *iter;
 	struct user *user;
 	struct ip_addr ip;
 
@@ -611,8 +617,8 @@ doveadm_cmd_user_list(struct doveadm_connection *conn, const char *const *args)
 		ip.family = 0;
 	}
 
-	iter = user_directory_iter_init(conn->dir->users);
-	while ((user = user_directory_iter_next(iter)) != NULL) {
+	iter = director_iterate_users_init(conn->dir);
+	while ((user = director_iterate_users_next(iter)) != NULL) {
 		if (ip.family == 0 ||
 		    net_ip_compare(&ip, &user->host->ip)) T_BEGIN {
 			unsigned int expire_time = user->timestamp +
@@ -624,7 +630,7 @@ doveadm_cmd_user_list(struct doveadm_connection *conn, const char *const *args)
 				net_ip2addr(&user->host->ip)));
 		} T_END;
 	}
-	user_directory_iter_deinit(&iter);
+	director_iterate_users_deinit(&iter);
 	o_stream_nsend(conn->output, "\n", 1);
 	return 1;
 }
@@ -649,8 +655,8 @@ doveadm_cmd_user_move(struct doveadm_connection *conn, const char *const *args)
 	}
 
 	if (str_to_uint(args[0], &username_hash) < 0)
-		username_hash = user_directory_get_username_hash(conn->dir->users, args[0]);
-	user = user_directory_lookup(conn->dir->users, username_hash);
+		username_hash = director_get_username_hash(conn->dir, args[0]);
+	user = user_directory_lookup(host->tag->users, username_hash);
 	if (user != NULL && USER_IS_BEING_KILLED(user)) {
 		o_stream_nsend_str(conn->output, "TRYAGAIN\n");
 		return 1;
