@@ -63,7 +63,7 @@ static void cmd_fs_get(int argc, char *argv[])
 	size_t size;
 	ssize_t ret;
 
-	doveadm_print_init(DOVEADM_PRINT_TYPE_FLOW);
+	doveadm_print_init(DOVEADM_PRINT_TYPE_PAGER);
 	doveadm_print_header("content", "content", DOVEADM_PRINT_HEADER_FLAG_HIDE_TITLE);
 
 	fs = cmd_fs_init(&argc, &argv, 1, cmd_fs_get);
@@ -78,11 +78,11 @@ static void cmd_fs_get(int argc, char *argv[])
 	i_assert(ret == -1);
 	if (input->stream_errno == ENOENT) {
 		i_error("%s doesn't exist: %s", fs_file_path(file),
-			fs_file_last_error(file));
+			i_stream_get_error(input));
 		doveadm_exit_code = DOVEADM_EX_NOTFOUND;
 	} else if (input->stream_errno != 0) {
 		i_error("read(%s) failed: %s", fs_file_path(file),
-			fs_file_last_error(file));
+			i_stream_get_error(input));
 		doveadm_exit_code = EX_TEMPFAIL;
 	}
 	i_stream_unref(&input);
