@@ -26,7 +26,7 @@ cmd_flags_run_box(struct flags_cmd_context *ctx,
 	struct mail_keywords *kw = NULL;
 
 	if (doveadm_mail_iter_init(&ctx->ctx, info, ctx->ctx.search_args,
-				   0, NULL, &iter) < 0)
+				   0, NULL, FALSE, &iter) < 0)
 		return -1;
 	box = doveadm_mail_iter_get_mailbox(iter);
 
