@@ -7,6 +7,7 @@
 
 struct doveadm_print_pager_header {
 	const char *title;
+	enum doveadm_print_header_flags flags;
 };
 
 struct doveadm_print_pager_context {
@@ -15,6 +16,7 @@ struct doveadm_print_pager_context {
 	unsigned int header_idx;
 
 	unsigned int streaming:1;
+	unsigned int first_page:1;
 };
 
 static struct doveadm_print_pager_context *ctx;
@@ -25,6 +27,7 @@ doveadm_print_pager_header(const struct doveadm_print_header *hdr)
 	struct doveadm_print_pager_header *fhdr;
 
 	fhdr = array_append_space(&ctx->headers);
+	fhdr->flags = hdr->flags;
 	fhdr->title = p_strdup(ctx->pool, hdr->title);
 }
 
@@ -32,7 +35,6 @@ static void pager_next_hdr(void)
 {
 	if (++ctx->header_idx == array_count(&ctx->headers)) {
 		ctx->header_idx = 0;
-		o_stream_nsend(doveadm_print_ostream, "\f\n", 2);
 	}
 }
 
@@ -41,8 +43,14 @@ static void doveadm_print_pager_print(const char *value)
 	const struct doveadm_print_pager_header *hdr =
 		array_idx(&ctx->headers, ctx->header_idx);
 
-	o_stream_nsend_str(doveadm_print_ostream, hdr->title);
-	o_stream_nsend(doveadm_print_ostream, ": ", 2);
+	if (ctx->header_idx == 0 && !ctx->first_page) {
+		o_stream_nsend(doveadm_print_ostream, "\f\n", 2);
+	}
+	ctx->first_page = FALSE;
+	if ((hdr->flags & DOVEADM_PRINT_HEADER_FLAG_HIDE_TITLE) == 0) {
+		o_stream_nsend_str(doveadm_print_ostream, hdr->title);
+		o_stream_nsend(doveadm_print_ostream, ": ", 2);
+	}
 	o_stream_nsend_str(doveadm_print_ostream, value);
 	o_stream_nsend(doveadm_print_ostream, "\n", 1);
 	pager_next_hdr();
@@ -56,8 +64,10 @@ doveadm_print_pager_print_stream(const unsigned char *value, size_t size)
 
 	if (!ctx->streaming) {
 		ctx->streaming = TRUE;
-		o_stream_nsend_str(doveadm_print_ostream, hdr->title);
-		o_stream_nsend(doveadm_print_ostream, ":\n", 2);
+		if ((hdr->flags & DOVEADM_PRINT_HEADER_FLAG_HIDE_TITLE) == 0) {
+			o_stream_nsend_str(doveadm_print_ostream, hdr->title);
+			o_stream_nsend(doveadm_print_ostream, ":\n", 2);
+		}
 	}
 	o_stream_nsend(doveadm_print_ostream, value, size);
 	if (size == 0) {
@@ -73,6 +83,7 @@ static void doveadm_print_pager_init(void)
 	pool = pool_alloconly_create("doveadm print pager", 1024);
 	ctx = p_new(pool, struct doveadm_print_pager_context, 1);
 	ctx->pool = pool;
+	ctx->first_page = TRUE;
 	p_array_init(&ctx->headers, pool, 16);
 }
 
@@ -91,7 +102,7 @@ static void doveadm_print_pager_deinit(void)
 }
 
 struct doveadm_print_vfuncs doveadm_print_pager_vfuncs = {
-	"pager",
+	DOVEADM_PRINT_TYPE_PAGER,
 
 	doveadm_print_pager_init,
 	doveadm_print_pager_deinit,
