@@ -57,7 +57,14 @@ sync_create_box(struct dsync_brain *brain, struct mailbox *box,
 	}
 
 	ret = memcmp(mailbox_guid, metadata.guid, sizeof(metadata.guid));
-	if (ret > 0) {
+
+	/* if THEIR guid is bigger than OUR guid, and we are not doing
+	   backup in either direction, OR GUID did not match and we are
+	   receiving backup, try change the mailbox GUID.
+	*/
+
+	if ((ret > 0 && !brain->backup_recv &&
+	     !brain->backup_send) || (ret != 0 && brain->backup_recv)) {
 		if (brain->debug) {
 			i_debug("brain %c: Changing mailbox %s GUID %s -> %s",
 				brain->master_brain ? 'M' : 'S',
@@ -122,18 +129,16 @@ int dsync_brain_mailbox_tree_sync_change(struct dsync_brain *brain,
 		ret = dsync_brain_mailbox_alloc(brain, change->mailbox_guid,
 						&box, &errstr, error_r);
 		if (ret < 0) {
-			i_error("Mailbox sync: Couldn't allocate mailbox GUID %s: %s",
+			i_error("Mailbox sync: Couldn't allocate mailbox %s GUID %s: %s",
+				change->full_name,
 				guid_128_to_string(change->mailbox_guid), errstr);
 			return -1;
 		}
 		if (ret == 0) {
-			if (brain->debug) {
-				i_debug("brain %c: Change during sync: "
-					"Mailbox GUID %s deletion conflict: %s",
-					brain->master_brain ? 'M' : 'S',
-					guid_128_to_string(change->mailbox_guid), errstr);
-			}
-			brain->changes_during_sync = TRUE;
+			dsync_brain_set_changes_during_sync(brain, t_strdup_printf(
+				"Mailbox %s GUID %s deletion conflict: %s",
+				change->full_name,
+				guid_128_to_string(change->mailbox_guid), errstr));
 			return 0;
 		}
 		break;
@@ -146,13 +151,9 @@ int dsync_brain_mailbox_tree_sync_change(struct dsync_brain *brain,
 		errstr = mailbox_list_get_last_error(change->ns->list, &error);
 		if (error == MAIL_ERROR_NOTFOUND ||
 		    error == MAIL_ERROR_EXISTS) {
-			if (brain->debug) {
-				i_debug("brain %c: Change during sync: "
-					"Mailbox %s mailbox_list_delete_dir conflict: %s",
-					brain->master_brain ? 'M' : 'S',
-					change->full_name, errstr);
-			}
-			brain->changes_during_sync = TRUE;
+			dsync_brain_set_changes_during_sync(brain, t_strdup_printf(
+				"Mailbox %s mailbox_list_delete_dir conflict: %s",
+				change->full_name, errstr));
 			return 0;
 		} else {
 			i_error("Mailbox sync: mailbox_list_delete_dir failed: %s",
@@ -213,14 +214,9 @@ int dsync_brain_mailbox_tree_sync_change(struct dsync_brain *brain,
 		    error == MAIL_ERROR_NOTFOUND) {
 			/* mailbox was already created or was already deleted.
 			   let the next sync figure out what to do */
-			if (brain->debug) {
-				i_debug("brain %c: Change during sync: "
-					"Mailbox %s %s conflict: %s",
-					brain->master_brain ? 'M' : 'S',
-					mailbox_get_vname(box),
-					func_name, errstr);
-			}
-			brain->changes_during_sync = TRUE;
+			dsync_brain_set_changes_during_sync(brain, t_strdup_printf(
+				"Mailbox %s %s conflict: %s",
+				mailbox_get_vname(box), func_name, errstr));
 			ret = 0;
 		} else {
 			i_error("Mailbox %s sync: %s failed: %s",
