@@ -196,16 +196,19 @@ dsync_ibc_recv_mail(struct dsync_ibc *ibc, struct dsync_mail **mail_r)
 }
 
 void dsync_ibc_send_finish(struct dsync_ibc *ibc, const char *error,
-			   enum mail_error mail_error)
+			   enum mail_error mail_error,
+			   bool require_full_resync)
 {
-	ibc->v.send_finish(ibc, error, mail_error);
+	ibc->v.send_finish(ibc, error, mail_error, require_full_resync);
 }
 
 enum dsync_ibc_recv_ret
 dsync_ibc_recv_finish(struct dsync_ibc *ibc, const char **error_r,
-		      enum mail_error *mail_error_r)
+		      enum mail_error *mail_error_r,
+		      bool *require_full_resync_r)
 {
-	return ibc->v.recv_finish(ibc, error_r, mail_error_r);
+	return ibc->v.recv_finish(ibc, error_r, mail_error_r,
+				  require_full_resync_r);
 }
 
 void dsync_ibc_close_mail_streams(struct dsync_ibc *ibc)
