@@ -63,6 +63,7 @@ struct dsync_mailbox_exporter {
 	unsigned int minimal_dmail_fill:1;
 	unsigned int return_all_mails:1;
 	unsigned int export_received_timestamps:1;
+	unsigned int export_virtual_sizes:1;
 	unsigned int no_hdr_hashes:1;
 };
 
@@ -284,12 +285,15 @@ search_add_save(struct dsync_mailbox_exporter *exporter, struct mail *mail)
 	const char *guid, *hdr_hash;
 	enum mail_fetch_field wanted_fields = MAIL_FETCH_GUID;
 	time_t received_timestamp = 0;
+	uoff_t virtual_size = (uoff_t)-1;
 	int ret;
 
 	/* update wanted fields in case we didn't already set them for the
 	   search */
 	if (exporter->export_received_timestamps)
 		wanted_fields |= MAIL_FETCH_RECEIVED_DATE;
+	if (exporter->export_virtual_sizes)
+		wanted_fields |= MAIL_FETCH_VIRTUAL_SIZE;
 	mail_add_temp_wanted_fields(mail, wanted_fields,
 				    exporter->wanted_headers);
 
@@ -306,12 +310,18 @@ search_add_save(struct dsync_mailbox_exporter *exporter, struct mail *mail)
 			received_timestamp = 1;
 		}
 	}
+	if (exporter->export_virtual_sizes) {
+		if (mail_get_virtual_size(mail, &virtual_size) < 0)
+			return dsync_mail_error(exporter, mail, "virtual-size");
+		i_assert(virtual_size != (uoff_t)-1);
+	}
 
 	change = export_save_change_get(exporter, mail->uid);
 	change->guid = *guid == '\0' ? "" :
 		p_strdup(exporter->pool, guid);
 	change->hdr_hash = p_strdup(exporter->pool, hdr_hash);
 	change->received_timestamp = received_timestamp;
+	change->virtual_size = virtual_size;
 	search_update_flag_changes(exporter, mail, change);
 
 	export_add_mail_instance(exporter, change, mail->seq);
@@ -508,6 +518,8 @@ dsync_mailbox_export_init(struct mailbox *box,
 		(flags & DSYNC_MAILBOX_EXPORTER_FLAG_MINIMAL_DMAIL_FILL) != 0;
 	exporter->export_received_timestamps =
 		(flags & DSYNC_MAILBOX_EXPORTER_FLAG_TIMESTAMPS) != 0;
+	exporter->export_virtual_sizes =
+		(flags & DSYNC_MAILBOX_EXPORTER_FLAG_VSIZES) != 0;
 	exporter->hdr_hash_version =
 		(flags & DSYNC_MAILBOX_EXPORTER_FLAG_HDR_HASH_V2) ? 2 : 1;
 	exporter->no_hdr_hashes =
