@@ -7,7 +7,8 @@ enum dsync_mailbox_exporter_flags {
 	DSYNC_MAILBOX_EXPORTER_FLAG_MINIMAL_DMAIL_FILL	= 0x04,
 	DSYNC_MAILBOX_EXPORTER_FLAG_TIMESTAMPS		= 0x08,
 	DSYNC_MAILBOX_EXPORTER_FLAG_HDR_HASH_V2		= 0x10,
-	DSYNC_MAILBOX_EXPORTER_FLAG_NO_HDR_HASHES	= 0x20
+	DSYNC_MAILBOX_EXPORTER_FLAG_NO_HDR_HASHES	= 0x20,
+	DSYNC_MAILBOX_EXPORTER_FLAG_VSIZES		= 0x40,
 };
 
 struct dsync_mailbox_exporter *
