@@ -341,9 +341,7 @@ void client_destroy(struct client *client, const char *reason)
 	
 	o_stream_destroy(&client->output);
 
-	net_disconnect(client->fd_in);
-	if (client->fd_in != client->fd_out)
-		net_disconnect(client->fd_out);
+	fd_close_maybe_stdio(&client->fd_in, &client->fd_out);
 
 	if (client->username != NULL)
 		i_free(client->username);
