@@ -44,6 +44,9 @@ static void imap_hibernate_write_cmd(struct client *client, string_t *cmd,
 				     const buffer_t *state, int fd_notify)
 {
 	struct stat peer_st;
+	const char *tag;
+
+	tag = client->command_queue == NULL ? NULL : client->command_queue->tag;
 
 	str_append_tabescaped(cmd, client->user->username);
 	str_append_c(cmd, '\t');
@@ -61,6 +64,10 @@ static void imap_hibernate_write_cmd(struct client *client, string_t *cmd,
 		str_append(cmd, "\tsession=");
 		str_append_tabescaped(cmd, client->session_id);
 	}
+	if (client->user->session_create_time != 0) {
+		str_printfa(cmd, "\tsession_created=%s",
+			    dec2str(client->user->session_create_time));
+	}
 	if (client->user->local_ip != NULL)
 		str_printfa(cmd, "\tlip=%s", net_ip2addr(client->user->local_ip));
 	if (client->user->remote_ip != NULL)
@@ -81,6 +88,8 @@ static void imap_hibernate_write_cmd(struct client *client, string_t *cmd,
 		str_printfa(cmd, "\tuid=%s", dec2str(client->user->uid));
 	if (client->user->gid != (gid_t)-1)
 		str_printfa(cmd, "\tgid=%s", dec2str(client->user->gid));
+	if (tag != NULL)
+		str_printfa(cmd, "\ttag=%s", tag);
 	str_append(cmd, "\tstats=");
 	str_append_tabescaped(cmd, client_stats(client));
 	if (client->command_queue != NULL &&
@@ -138,7 +147,7 @@ static int imap_hibernate_process_read(int fd, const char *path)
 
 static int
 imap_hibernate_process_send(struct client *client,
-			    const buffer_t *state, int fd_notify)
+			    const buffer_t *state, int fd_notify, int *fd_r)
 {
 	string_t *cmd = t_str_new(512);
 	const char *path;
@@ -147,6 +156,8 @@ imap_hibernate_process_send(struct client *client,
 
 	i_assert(state->used > 0);
 
+	*fd_r = -1;
+
 	path = t_strconcat(client->user->set->base_dir,
 			   "/"IMAP_HIBERNATE_SOCKET_NAME, NULL);
 	fd = net_connect_unix_with_retries(path, 1000);
@@ -169,8 +180,12 @@ imap_hibernate_process_send(struct client *client,
 			ret = imap_hibernate_process_read(fd, path);
 	}
 	alarm(0);
-	net_disconnect(fd);
-	return ret < 0 ? -1 : 0;
+	if (ret < 0) {
+		net_disconnect(fd);
+		return -1;
+	}
+	*fd_r = fd;
+	return 0;
 }
 
 bool imap_client_hibernate(struct client **_client)
@@ -178,7 +193,7 @@ bool imap_client_hibernate(struct client **_client)
 	struct client *client = *_client;
 	buffer_t *state;
 	const char *error;
-	int ret, fd_notify = -1;
+	int ret, fd_notify = -1, fd_hibernate = -1;
 
 	if (client->fd_in != client->fd_out) {
 		/* we won't try to hibernate stdio clients */
@@ -215,7 +230,7 @@ bool imap_client_hibernate(struct client **_client)
 		}
 	}
 	if (ret > 0) {
-		if (imap_hibernate_process_send(client, state, fd_notify) < 0)
+		if (imap_hibernate_process_send(client, state, fd_notify, &fd_hibernate) < 0)
 			ret = -1;
 	}
 	if (fd_notify != -1)
@@ -227,6 +242,12 @@ bool imap_client_hibernate(struct client **_client)
 		client_destroy(client, NULL);
 		*_client = NULL;
 	}
+	/* notify imap-hibernate that we're done by closing the connection.
+	   do this only after client is destroyed. this way imap-hibernate
+	   won't try to launch another imap process too early and cause
+	   problems (like sending duplicate session ID to stats process) */
+	if (fd_hibernate != -1)
+		net_disconnect(fd_hibernate);
 	buffer_free(&state);
 	return ret > 0;
 }
