@@ -400,9 +400,7 @@ static void client_default_destroy(struct client *client, const char *reason)
 	timeout_remove(&client->to_idle);
 
 	/* i/ostreams are already closed at this stage, so fd can be closed */
-	net_disconnect(client->fd_in);
-	if (client->fd_in != client->fd_out)
-		net_disconnect(client->fd_out);
+	fd_close_maybe_stdio(&client->fd_in, &client->fd_out);
 
 	/* Free the user after client is already disconnected. It may start
 	   some background work like autoexpunging. */
