@@ -26,6 +26,8 @@ struct imap_master_input {
 	buffer_t *client_output;
 	/* IMAP connection state */
 	buffer_t *state;
+	/* command tag */
+	const char *tag;
 
 	dev_t peer_dev;
 	ino_t peer_ino;
@@ -112,6 +114,12 @@ imap_master_client_parse_input(const char *const *args, pool_t pool,
 			}
 		} else if (strcmp(key, "session") == 0) {
 			input_r->session_id = value;
+		} else if (strcmp(key, "session_created") == 0) {
+			if (str_to_time(value, &input_r->session_create_time) < 0) {
+				*error_r = t_strdup_printf(
+					"Invalid session_created value: %s", value);
+				return -1;
+			}
 		} else if (strcmp(key, "userdb_fields") == 0) {
 			input_r->userdb_fields =
 				t_strsplit_tabescaped(value);
@@ -136,6 +144,8 @@ imap_master_client_parse_input(const char *const *args, pool_t pool,
 					"Invalid state base64 value: %s", value);
 				return -1;
 			}
+		} else if (strcmp(key, "tag") == 0) {
+			master_input_r->tag = t_strdup(value);
 		} else if (strcmp(key, "bad-done") == 0) {
 			master_input_r->state_import_bad_idle_done = TRUE;
 		} else if (strcmp(key, "idle-continue") == 0) {
@@ -203,6 +213,10 @@ imap_master_client_input_args(struct connection *conn, const char *const *args,
 		i_close_fd(&fd_client);
 		return -1;
 	}
+	/* Send a success notification before we start anything that lasts
+	   potentially a long time. imap-hibernate process is waiting for us
+	   to answer. Even if we fail later, we log the error anyway. */
+	o_stream_send_str(conn->output, "+\n");
 
 	/* NOTE: before client_create_from_input() on failures we need to close
 	   fd_client, but afterward it gets closed by client_destroy() */
@@ -211,9 +225,6 @@ imap_master_client_input_args(struct connection *conn, const char *const *args,
 	if (ret < 0) {
 		i_error("imap-master(%s): Failed to create client: %s",
 			input.username, error);
-		o_stream_send_str(conn->output, t_strdup_printf(
-			"-Failed to create client: %s\n", error));
-		master_service_client_connection_destroyed(master_service);
 		i_close_fd(&fd_client);
 		return -1;
 	}
@@ -231,8 +242,6 @@ imap_master_client_input_args(struct connection *conn, const char *const *args,
 		i_error("imap-master: Couldn't add %"PRIuSIZE_T
 			" bytes to client's input stream",
 			master_input.client_input->used);
-		o_stream_send_str(conn->output,
-				  "-Couldn't add client input\n");
 		client_destroy(imap_client, "Client initialization failed");
 		return -1;
 	}
@@ -248,6 +257,9 @@ imap_master_client_input_args(struct connection *conn, const char *const *args,
 		return -1;
 	}
 
+	if (master_input.tag != NULL)
+		imap_state_import_idle_cmd_tag(imap_client, master_input.tag);
+
 	/* make sure all pending input gets handled */
 	i_assert(imap_client->to_delayed_input == NULL);
 	if (master_input.client_input->used > 0) {
@@ -255,7 +267,6 @@ imap_master_client_input_args(struct connection *conn, const char *const *args,
 			timeout_add(0, client_input, imap_client);
 	}
 
-	o_stream_send_str(conn->output, "+\n");
 	imap_refresh_proctitle();
 	/* we'll always disconnect the client afterwards */
 	return -1;
