@@ -9,7 +9,7 @@
 #include "str-sanitize.h"
 #include "imap-util.h"
 #include "mail-search-build.h"
-#include "mail-storage.h"
+#include "mail-storage-private.h"
 #include "mailbox-recent-flags.h"
 #include "imap-client.h"
 #include "imap-fetch.h"
@@ -402,7 +402,9 @@ import_send_expunges(struct client *client,
 		*error_r = mailbox_get_last_error(client->mailbox, NULL);
 		ret = -1;
 	} else if (seq != state->messages) {
-		*error_r = "Message count mismatch after handling expunges";
+		*error_r = t_strdup_printf("Message count mismatch after "
+					   "handling expunges (%u != %u)",
+					   seq, state->messages);
 		ret = -1;
 	}
 	(void)mailbox_transaction_commit(&trans);
@@ -411,11 +413,15 @@ import_send_expunges(struct client *client,
 
 	seqs = array_get(&expunged_seqs, &expunge_count);
 	if (client->messages_count + expunge_count < state->messages) {
-		*error_r = "Message count too low after handling expunges";
+		*error_r = t_strdup_printf("Message count too low after "
+					   "handling expunges (%u < %u)",
+					   client->messages_count + expunge_count,
+					   state->messages);
 		return -1;
 	}
 	if (crc != state->uids_crc32) {
-		*error_r = "Message UIDs CRC32 mismatch";
+		*error_r = t_strdup_printf("Message UIDs CRC32 mismatch (%u != %u)",
+					   crc, state->uids_crc32);
 		return -1;
 	}
 
@@ -585,32 +591,42 @@ import_state_mailbox_open(struct client *client,
 	/* verify that this still looks like the same mailbox */
 	if (mailbox_get_metadata(box, MAILBOX_METADATA_GUID, &metadata) < 0) {
 		*error_r = mailbox_get_last_error(box, NULL);
+		mailbox_free(&box);
 		return -1;
 	}
 	if (!guid_128_equals(metadata.guid, state->mailbox_guid)) {
-		*error_r = "Mailbox GUID has changed";
+		*error_r = t_strdup_printf("Mailbox GUID has changed %s->%s",
+					   guid_128_to_string(state->mailbox_guid),
+					   guid_128_to_string(metadata.guid));
+		mailbox_free(&box);
 		return -1;
 	}
 	mailbox_get_open_status(box, STATUS_UIDVALIDITY | STATUS_UIDNEXT |
 				STATUS_HIGHESTMODSEQ | STATUS_RECENT |
 				STATUS_KEYWORDS, &status);
 	if (status.uidvalidity != state->uidvalidity) {
-		*error_r = "Mailbox UIDVALIDITY has changed";
+		*error_r = t_strdup_printf("Mailbox UIDVALIDITY has changed %u->%u",
+					    state->uidvalidity, status.uidvalidity);
+		mailbox_free(&box);
 		return -1;
 	}
 	if (status.uidnext < state->uidnext) {
-		*error_r = "Mailbox UIDNEXT shrank";
+		*error_r = t_strdup_printf("Mailbox UIDNEXT shrank %u -> %u",
+					   state->uidnext, status.uidnext);
+		mailbox_free(&box);
 		return -1;
 	}
 	if (status.highest_modseq < state->highest_modseq) {
-		*error_r = "Mailbox HIGHESTMODSEQ shrank";
+		*error_r = t_strdup_printf("Mailbox HIGHESTMODSEQ shrank %llu -> %llu",
+					   (unsigned long long)state->highest_modseq,
+					   (unsigned long long)status.highest_modseq);
+		mailbox_free(&box);
 		return -1;
 	}
 
 	client->mailbox = box;
 	client->mailbox_examined = state->examined;
 	client->messages_count = status.messages;
-	client->recent_count = status.recent;
 	client->uidvalidity = status.uidvalidity;
 	client->notify_uidnext = status.uidnext;
 
@@ -618,9 +634,23 @@ import_state_mailbox_open(struct client *client,
 		return -1;
 	i_assert(expunge_count <= state->messages);
 	if (state->messages - expunge_count > client->messages_count) {
-		*error_r = "Mailbox message count shrank";
+		*error_r = t_strdup_printf("Mailbox message count shrank %u -> %u",
+					   client->messages_count,
+					   state->messages - expunge_count);
 		return -1;
 	}
+
+	client_update_mailbox_flags(client, status.keywords);
+	array_foreach(&state->recent_uids, range) {
+		for (uid = range->seq1; uid <= range->seq2; uid++) {
+			uint32_t seq;
+
+			if (mail_index_lookup_seq(box->view, uid, &seq))
+				mailbox_recent_flags_set_uid_forced(box, uid);
+		}
+	}
+	client->recent_count = mailbox_recent_flags_count(box);
+
 	if (state->messages - expunge_count < client->messages_count) {
 		/* new messages arrived */
 		client_send_line(client,
@@ -629,11 +659,6 @@ import_state_mailbox_open(struct client *client,
 			t_strdup_printf("* %u RECENT", client->recent_count));
 	}
 
-	client_update_mailbox_flags(client, status.keywords);
-	array_foreach(&state->recent_uids, range) {
-		for (uid = range->seq1; uid <= range->seq2; uid++)
-			mailbox_recent_flags_set_uid_forced(box, uid);
-	}
 	if (array_count(status.keywords) == state->keywords_count &&
 	    mailbox_status_keywords_crc32(&status) == state->keywords_crc32) {
 		/* no changes to keywords */
