@@ -153,7 +153,7 @@ static ssize_t i_stream_lzma_read(struct istream_private *stream)
 		return -1;
 	case LZMA_OPTIONS_ERROR:
 		lzma_read_error(zstream, "Unsupported xz options");
-		stream->istream.stream_errno = EINVAL;
+		stream->istream.stream_errno = EIO;
 		return -1;
 	case LZMA_MEM_ERROR:
 		i_fatal_status(FATAL_OUTOFMEM, "lzma.read(%s): Out of memory",
@@ -168,7 +168,7 @@ static ssize_t i_stream_lzma_read(struct istream_private *stream)
 	default:
 		lzma_read_error(zstream, t_strdup_printf(
 			"lzma_code() failed with %d", ret));
-		stream->istream.stream_errno = EINVAL;
+		stream->istream.stream_errno = EIO;
 		return -1;
 	}
 	if (out_size == 0) {
@@ -235,6 +235,8 @@ i_stream_lzma_seek(struct istream_private *stream, uoff_t v_offset, bool mark)
 		stream->pos = stream->skip;
 	} else {
 		/* read and cache forward */
+		ssize_t ret = -1;
+
 		do {
 			size_t avail = stream->pos - stream->skip;
 
@@ -246,7 +248,8 @@ i_stream_lzma_seek(struct istream_private *stream, uoff_t v_offset, bool mark)
 			}
 
 			i_stream_skip(&stream->istream, avail);
-		} while (i_stream_read(&stream->istream) >= 0);
+		} while ((ret = i_stream_read(&stream->istream)) > 0);
+		i_assert(ret == -1);
 
 		if (stream->istream.v_offset != v_offset) {
 			/* some failure, we've broken it */
@@ -289,11 +292,13 @@ i_stream_lzma_stat(struct istream_private *stream, bool exact)
 
 	if (zstream->stream_size == (uoff_t)-1) {
 		uoff_t old_offset = stream->istream.v_offset;
+		ssize_t ret;
 
 		do {
 			size = i_stream_get_data_size(&stream->istream);
 			i_stream_skip(&stream->istream, size);
-		} while (i_stream_read(&stream->istream) > 0);
+		} while ((ret = i_stream_read(&stream->istream)) > 0);
+		i_assert(ret == -1);
 
 		i_stream_seek(&stream->istream, old_offset);
 		if (zstream->stream_size == (uoff_t)-1)
