@@ -6,7 +6,7 @@
 #define DEFAULT_DICT_SERVER_SOCKET_FNAME "dict"
 
 #define DICT_CLIENT_PROTOCOL_MAJOR_VERSION 2
-#define DICT_CLIENT_PROTOCOL_MINOR_VERSION 0
+#define DICT_CLIENT_PROTOCOL_MINOR_VERSION 1
 
 #define DICT_CLIENT_MAX_LINE_LENGTH (64*1024)
 
@@ -29,11 +29,16 @@ enum dict_protocol_cmd {
 };
 
 enum dict_protocol_reply {
+	DICT_PROTOCOL_REPLY_ERROR = -1,
+
 	DICT_PROTOCOL_REPLY_OK = 'O', /* <value> */
 	DICT_PROTOCOL_REPLY_NOTFOUND = 'N',
 	DICT_PROTOCOL_REPLY_FAIL = 'F',
 	DICT_PROTOCOL_REPLY_WRITE_UNCERTAIN = 'W',
-	DICT_PROTOCOL_REPLY_ASYNC_COMMIT = 'A'
+	DICT_PROTOCOL_REPLY_ASYNC_COMMIT = 'A',
+	DICT_PROTOCOL_REPLY_ITER_FINISHED = '\0',
+	DICT_PROTOCOL_REPLY_ASYNC_ID = '*',
+	DICT_PROTOCOL_REPLY_ASYNC_REPLY = '+',
 };
 
 const char *dict_client_escape(const char *src);
