@@ -650,20 +650,15 @@ memcached_ascii_transaction_commit(struct dict_transaction_context *_ctx,
 struct dict dict_driver_memcached_ascii = {
 	.name = "memcached_ascii",
 	{
-		memcached_ascii_dict_init,
-		memcached_ascii_dict_deinit,
-		NULL,
-		memcached_ascii_dict_lookup,
-		NULL,
-		NULL,
-		NULL,
-		memcached_ascii_transaction_init,
-		memcached_ascii_transaction_commit,
-		dict_transaction_memory_rollback,
-		dict_transaction_memory_set,
-		dict_transaction_memory_unset,
-		dict_transaction_memory_append,
-		dict_transaction_memory_atomic_inc,
-		NULL
+		.init = memcached_ascii_dict_init,
+		.deinit = memcached_ascii_dict_deinit,
+		.lookup = memcached_ascii_dict_lookup,
+		.transaction_init = memcached_ascii_transaction_init,
+		.transaction_commit = memcached_ascii_transaction_commit,
+		.transaction_rollback = dict_transaction_memory_rollback,
+		.set = dict_transaction_memory_set,
+		.unset = dict_transaction_memory_unset,
+		.append = dict_transaction_memory_append,
+		.atomic_inc = dict_transaction_memory_atomic_inc,
 	}
 };
