@@ -13,6 +13,10 @@ struct hash_method;
    This can be later on used to optimize reads by setting it before reading
    the file. */
 #define FS_METADATA_OBJECTID FS_METADATA_INTERNAL_PREFIX"ObjectID"
+/* Calling this before fs_write_stream_finish() allows renaming the filename.
+   This can be useful if you don't know the final filename before writing it
+   (e.g. filename contains the file size). */
+#define FS_METADATA_WRITE_FNAME FS_METADATA_INTERNAL_PREFIX"WriteFilename"
 
 enum fs_properties {
 	FS_PROPERTY_METADATA	= 0x01,
@@ -36,7 +40,9 @@ enum fs_properties {
 	/* Backend support asynchronous file operations. */
 	FS_PROPERTY_ASYNC		= 0x800,
 	/* Backend supports FS_ITER_FLAG_OBJECTIDS. */
-	FS_PROPERTY_OBJECTIDS		= 0x1000
+	FS_PROPERTY_OBJECTIDS		= 0x1000,
+	/* fs_copy() is fast even when file's metadata is changed */
+	FS_PROPERTY_FASTCOPY_CHANGED_METADATA = 0x2000,
 };
 
 enum fs_open_mode {
@@ -93,6 +99,23 @@ enum fs_iter_flags {
 	FS_ITER_FLAG_NOCACHE	= 0x08
 };
 
+enum fs_op {
+	FS_OP_WAIT,
+	FS_OP_METADATA,
+	FS_OP_PREFETCH,
+	FS_OP_READ,
+	FS_OP_WRITE,
+	FS_OP_LOCK,
+	FS_OP_EXISTS,
+	FS_OP_STAT,
+	FS_OP_COPY,
+	FS_OP_RENAME,
+	FS_OP_DELETE,
+	FS_OP_ITER,
+
+	FS_OP_COUNT
+};
+
 struct fs_settings {
 	/* Username and session ID are mainly used for debugging/logging,
 	   but may also be useful for other purposes if they exist (they
@@ -120,6 +143,8 @@ struct fs_settings {
 
 	/* Enable debugging */
 	bool debug;
+	/* Enable timing statistics */
+	bool enable_timing;
 };
 
 struct fs_stats {
@@ -153,6 +178,13 @@ struct fs_stats {
 	unsigned int rename_count;
 	/* Number of fs_iter_init() calls. */
 	unsigned int iter_count;
+
+	/* Number of bytes written by fs_write*() calls. */
+	uint64_t write_bytes;
+
+	/* Cumulative sum of usecs spent on calls - set only if
+	   fs_settings.enable_timing=TRUE */
+	struct timing *timings[FS_OP_COUNT];
 };
 
 struct fs_metadata {
@@ -166,8 +198,12 @@ typedef void fs_file_async_callback_t(void *context);
 int fs_init(const char *driver, const char *args,
 	    const struct fs_settings *set,
 	    struct fs **fs_r, const char **error_r);
+/* same as fs_unref() */
 void fs_deinit(struct fs **fs);
 
+void fs_ref(struct fs *fs);
+void fs_unref(struct fs **fs);
+
 /* Returns the parent filesystem (if this is a wrapper fs) or NULL if
    there's no parent. */
 struct fs *fs_get_parent(struct fs *fs);
@@ -227,16 +263,23 @@ struct istream *fs_read_stream(struct fs_file *file, size_t max_buffer_size);
 int fs_write(struct fs_file *file, const void *data, size_t size);
 
 /* Write to file via output stream. The stream will be destroyed by
-   fs_write_stream_finish/abort. */
+   fs_write_stream_finish/abort. The returned ostream is already corked and
+   it doesn't need to be uncorked. */
 struct ostream *fs_write_stream(struct fs_file *file);
-/* Finish writing via stream. The file will be created/replaced/appended only
+/* Finish writing via stream, calling also o_stream_nfinish() on the stream and
+   handling any pending errors. The file will be created/replaced/appended only
    after this call, same as with fs_write(). Anything written to the stream
    won't be visible earlier. Returns 1 if ok, 0 if async write isn't finished
    yet (retry calling fs_write_stream_finish_async()), -1 if error */
 int fs_write_stream_finish(struct fs_file *file, struct ostream **output);
 int fs_write_stream_finish_async(struct fs_file *file);
-/* Abort writing via stream. Anything written to the stream is discarded. */
+/* Abort writing via stream. Anything written to the stream is discarded.
+   o_stream_ignore_last_errors() is called on the output stream so the caller
+   doesn't need to do it. This must not be called after
+   fs_write_stream_finish(), i.e. it can't be used to abort a pending async
+   write. */
 void fs_write_stream_abort(struct fs_file *file, struct ostream **output);
+void fs_write_stream_abort_error(struct fs_file *file, struct ostream **output, const char *error_fmt, ...) ATTR_FORMAT(3, 4);
 
 /* Set a hash to the following write. The storage can then verify that the
    input data matches the specified hash, or fail if it doesn't. Typically
@@ -253,6 +296,10 @@ void fs_file_set_async_callback(struct fs_file *file,
    It's an error to call this when there are no pending async operations.
    Returns 0 if ok, -1 if timed out. */
 int fs_wait_async(struct fs *fs);
+/* Switch the fs to the current ioloop. This can be used to do fs_wait_async()
+   among other IO work. Returns TRUE if there is actually some work that can
+   be waited on. */
+bool fs_switch_ioloop(struct fs *fs) ATTR_NOWARN_UNUSED_RESULT;
 
 /* Returns 1 if file exists, 0 if not, -1 if error occurred. */
 int fs_exists(struct fs_file *file);
@@ -262,6 +309,10 @@ int fs_delete(struct fs_file *file);
 /* Returns 0 if ok, -1 if error occurred (e.g. errno=ENOENT).
    All fs backends may not support all stat fields. */
 int fs_stat(struct fs_file *file, struct stat *st_r);
+/* Get number of links to the file. This is the same as using fs_stat()'s
+   st_nlinks field, except not all backends support returning it via fs_stat().
+   Returns 0 if ok, -1 if error occurred. */
+int fs_get_nlinks(struct fs_file *file, nlink_t *nlinks_r);
 /* Copy an object with possibly updated metadata. Destination parent
    directories are created automatically. Returns 0 if ok, -1 if error
    occurred. */
@@ -302,4 +353,8 @@ bool fs_iter_have_more(struct fs_iter *iter);
    filesystem whose stats you want to see. */
 const struct fs_stats *fs_get_stats(struct fs *fs);
 
+/* Helper functions to count number of usecs for read/write operations. */
+uint64_t fs_stats_get_read_usecs(const struct fs_stats *stats);
+uint64_t fs_stats_get_write_usecs(const struct fs_stats *stats);
+
 #endif
