@@ -152,6 +152,16 @@ http_server_connection_timeout_reset(struct http_server_connection *conn)
 		timeout_reset(conn->to_idle);
 }
 
+bool http_server_connection_shut_down(struct http_server_connection *conn)
+{
+	if (conn->request_queue_head == NULL ||
+		conn->request_queue_head->state == HTTP_SERVER_REQUEST_STATE_NEW) {
+		http_server_connection_close(&conn, "Server shutting down");
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static void http_server_connection_ready(struct http_server_connection *conn)
 {
 	struct stat st;
@@ -410,17 +420,24 @@ static bool
 http_server_connection_pipeline_is_full(struct http_server_connection *conn)
 {
 	return (conn->request_queue_count >=
-			conn->server->set.max_pipelined_requests);
+			conn->server->set.max_pipelined_requests ||
+			conn->server->shutting_down);
 }
 
 static void
 http_server_connection_pipeline_handle_full(
 	struct http_server_connection *conn)
 {
-	http_server_connection_debug(conn,
-		"Pipeline full (%u requests pending; %u maximum)",
-		conn->request_queue_count,
-		conn->server->set.max_pipelined_requests);
+	if (conn->server->shutting_down) {
+		http_server_connection_debug(conn,
+			"Pipeline full (%u requests pending; server shutting down)",
+			conn->request_queue_count);
+	} else {
+		http_server_connection_debug(conn,
+			"Pipeline full (%u requests pending; %u maximum)",
+			conn->request_queue_count,
+			conn->server->set.max_pipelined_requests);
+	}
 	http_server_connection_input_halt(conn);
 }
 
@@ -524,6 +541,12 @@ static void http_server_connection_input(struct connection *_conn)
 	bool cont;
 	int ret;
 
+	if (conn->server->shutting_down) {
+		if (!http_server_connection_shut_down(conn))
+			http_server_connection_pipeline_handle_full(conn);
+		return;
+	}
+
 	i_assert(!conn->in_req_callback);
 	i_assert(!conn->input_broken && conn->incoming_payload == NULL);
 	i_assert(!conn->close_indicated);
@@ -895,7 +918,8 @@ static int http_server_connection_send_responses(
 
 	/* accept more requests if possible */
 	if (conn->incoming_payload == NULL &&
-		conn->request_queue_count < conn->server->set.max_pipelined_requests) {
+		conn->request_queue_count < conn->server->set.max_pipelined_requests &&
+		!conn->server->shutting_down) {
 		http_server_connection_input_resume(conn);
 		return 1;
 	}
@@ -967,6 +991,10 @@ int http_server_connection_output(struct http_server_connection *conn)
 		}
 	}
 
+	if (conn->server->shutting_down &&
+		http_server_connection_shut_down(conn))
+		return 1;
+
 	if (!http_server_connection_pipeline_is_full(conn)) {
 		http_server_connection_input_resume(conn);
 		if (pipeline_was_full && conn->conn.io != NULL)
@@ -1018,6 +1046,8 @@ http_server_connection_create(struct http_server *server,
 	in_port_t port;
 	const char *name;
 
+	i_assert(!server->shutting_down);
+
 	conn = i_new(struct http_server_connection, 1);
 	conn->refcount = 1;
 	conn->id = id++;
@@ -1042,6 +1072,7 @@ http_server_connection_create(struct http_server *server,
 			i_error("net_set_recv_buffer_size(%"PRIuSIZE_T") failed: %m",
 				set->socket_recv_buffer_size);
 	}
+	(void)net_set_tcp_nodelay(fd_out, TRUE);
 
 	/* get a name for this connection */
 	if (fd_in != fd_out || net_getpeername(fd_in, &addr, &port) < 0) {
