@@ -5,6 +5,7 @@
 #include "hostpid.h"
 #include "ioloop.h"
 #include "istream.h"
+#include "istream-chain.h"
 #include "ostream.h"
 #include "time-util.h"
 #include "connection.h"
@@ -29,8 +30,9 @@ struct server_connection {
 };
 
 typedef void (*test_server_init_t)(unsigned int index);
-typedef void (*test_client_init_t)
+typedef bool (*test_client_init_t)
 	(const struct http_client_settings *client_set);
+typedef void (*test_dns_init_t)(void);
 
 /*
  * State
@@ -42,6 +44,9 @@ static in_port_t *bind_ports = 0;
 static struct ioloop *ioloop;
 static bool debug = FALSE;
 
+/* dns */
+static pid_t dns_pid = (pid_t)-1;
+
 /* server */
 static struct io *io_listen;
 static int fd_listen = -1;
@@ -74,7 +79,8 @@ static void test_run_client_server(
 	const struct http_client_settings *client_set,
 	test_client_init_t client_test,
 	test_server_init_t server_test,
-	unsigned int server_tests_count)
+	unsigned int server_tests_count,
+	test_dns_init_t dns_test)
 	ATTR_NULL(3);
 
 /*
@@ -104,7 +110,7 @@ test_client_host_lookup_failed_response(
 	}
 }
 
-static void
+static bool
 test_client_host_lookup_failed(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -124,6 +130,8 @@ test_client_host_lookup_failed(const struct http_client_settings *client_set)
 		"GET", "host.in-addr.arpa", "/host-lookup-failed2.txt",
 		test_client_host_lookup_failed_response, ctx);
 	http_client_request_submit(hreq);
+
+	return TRUE;
 }
 
 /* test */
@@ -137,7 +145,7 @@ static void test_host_lookup_failed(void)
 	test_begin("host lookup failed");
 	test_run_client_server(&http_client_set,
 		test_client_host_lookup_failed,
-		NULL, 0);
+		NULL, 0, NULL);
 	test_end();
 }
 
@@ -176,7 +184,7 @@ test_client_connection_refused_response(
 	}
 }
 
-static void
+static bool
 test_client_connection_refused(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -198,6 +206,8 @@ test_client_connection_refused(const struct http_client_settings *client_set)
 		test_client_connection_refused_response, ctx);
 	http_client_request_set_port(hreq, bind_ports[0]);
 	http_client_request_submit(hreq);
+
+	return TRUE;
 }
 
 /* test */
@@ -211,7 +221,8 @@ static void test_connection_refused(void)
 	test_begin("connection refused");
 	test_run_client_server(&http_client_set,
 		test_client_connection_refused,
-		test_server_connection_refused, 1);
+		test_server_connection_refused, 1,
+		NULL);
 	test_end();
 }
 
@@ -242,7 +253,7 @@ test_client_connection_timed_out_response(
 	}
 }
 
-static void
+static bool
 test_client_connection_timed_out(
 	const struct http_client_settings *client_set)
 {
@@ -263,6 +274,8 @@ test_client_connection_timed_out(
 		"GET", "192.168.0.0", "/connection-timed-out2.txt",
 		test_client_connection_timed_out_response, ctx);
 	http_client_request_submit(hreq);
+
+	return TRUE;
 }
 
 /* test */
@@ -278,7 +291,7 @@ static void test_connection_timed_out(void)
 	test_begin("connection timed out");
 	test_run_client_server(&http_client_set,
 		test_client_connection_timed_out,
-		NULL, 0);
+		NULL, 0, NULL);
 	test_end();
 }
 
@@ -365,7 +378,7 @@ test_client_invalid_redirect_response(
 	io_loop_stop(ioloop);
 }
 
-static void
+static bool
 test_client_invalid_redirect(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -377,6 +390,8 @@ test_client_invalid_redirect(const struct http_client_settings *client_set)
 		test_client_invalid_redirect_response, NULL);
 	http_client_request_set_port(hreq, bind_ports[0]);
 	http_client_request_submit(hreq);
+
+	return TRUE;
 }
 
 /* test */
@@ -391,21 +406,24 @@ static void test_invalid_redirect(void)
 	http_client_set.max_redirects = 0;
 	test_run_client_server(&http_client_set,
 		test_client_invalid_redirect,
-		test_server_invalid_redirect1, 1);
+		test_server_invalid_redirect1, 1,
+		NULL);
 	test_end();
 
 	test_begin("invalid redirect: bad location");
 	http_client_set.max_redirects = 1;
 	test_run_client_server(&http_client_set,
 		test_client_invalid_redirect,
-		test_server_invalid_redirect2, 1);
+		test_server_invalid_redirect2, 1,
+		NULL);
 	test_end();
 
 	test_begin("invalid redirect: too many");
 	http_client_set.max_redirects = 1;
 	test_run_client_server(&http_client_set,
 		test_client_invalid_redirect,
-		test_server_invalid_redirect3, 3);
+		test_server_invalid_redirect3, 3,
+		NULL);
 	test_end();
 }
 
@@ -452,7 +470,7 @@ test_client_unseekable_redirect_response(
 	io_loop_stop(ioloop);
 }
 
-static void
+static bool
 test_client_unseekable_redirect(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -470,7 +488,8 @@ test_client_unseekable_redirect(const struct http_client_settings *client_set)
 	http_client_request_set_payload(hreq, input, FALSE);
 	http_client_request_submit(hreq);
 
-	i_stream_unref(&input);	
+	i_stream_unref(&input);
+	return TRUE;
 }
 
 /* test */
@@ -485,7 +504,8 @@ static void test_unseekable_redirect(void)
 	test_begin("unseekable redirect");
 	test_run_client_server(&http_client_set,
 		test_client_unseekable_redirect,
-		test_server_unseekable_redirect, 2);
+		test_server_unseekable_redirect, 2,
+		NULL);
 	test_end();
 }
 
@@ -522,7 +542,7 @@ test_client_unseekable_retry_response(
 	io_loop_stop(ioloop);
 }
 
-static void
+static bool
 test_client_unseekable_retry(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -540,7 +560,8 @@ test_client_unseekable_retry(const struct http_client_settings *client_set)
 	http_client_request_set_payload(hreq, input, FALSE);
 	http_client_request_submit(hreq);
 
-	i_stream_unref(&input);	
+	i_stream_unref(&input);
+	return TRUE;
 }
 
 /* test */
@@ -555,7 +576,8 @@ static void test_unseekable_retry(void)
 	test_begin("unseekable retry");
 	test_run_client_server(&http_client_set,
 		test_client_unseekable_retry,
-		test_server_unseekable_retry, 2);
+		test_server_unseekable_retry, 2,
+		NULL);
 	test_end();
 }
 
@@ -598,7 +620,7 @@ test_client_broken_payload_response(
 	io_loop_stop(ioloop);
 }
 
-static void
+static bool
 test_client_broken_payload(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -618,7 +640,8 @@ test_client_broken_payload(const struct http_client_settings *client_set)
 	http_client_request_set_payload(hreq, input, FALSE);
 	http_client_request_submit(hreq);
 
-	i_stream_unref(&input);	
+	i_stream_unref(&input);
+	return TRUE;	
 }
 
 /* test */
@@ -632,7 +655,8 @@ static void test_broken_payload(void)
 	test_begin("broken payload");
 	test_run_client_server(&http_client_set,
 		test_client_broken_payload,
-		test_server_broken_payload, 1);
+		test_server_broken_payload, 1,
+		NULL);
 	test_end();
 }
 
@@ -710,7 +734,7 @@ test_client_connection_lost_response(
 	}
 }
 
-static void
+static bool
 test_client_connection_lost(const struct http_client_settings *client_set)
 {
 	static const char payload[] =
@@ -748,7 +772,8 @@ test_client_connection_lost(const struct http_client_settings *client_set)
 	http_client_request_set_port(hreq, bind_ports[0]);
 	http_client_request_submit(hreq);
 
-	i_stream_unref(&input);	
+	i_stream_unref(&input);
+	return TRUE;
 }
 
 /* test */
@@ -765,21 +790,24 @@ static void test_connection_lost(void)
 	http_client_set.max_attempts = 1;
 	test_run_client_server(&http_client_set,
 		test_client_connection_lost,
-		test_server_connection_lost, 1);
+		test_server_connection_lost, 1,
+		NULL);
 	test_end();
 
 	test_begin("connection lost: two attempts");
 	http_client_set.max_attempts = 2;
 	test_run_client_server(&http_client_set,
 		test_client_connection_lost,
-		test_server_connection_lost, 1);
+		test_server_connection_lost, 1,
+		NULL);
 	test_end();
 
 	test_begin("connection lost: three attempts");
 	http_client_set.max_attempts = 3;
 	test_run_client_server(&http_client_set,
 		test_client_connection_lost,
-		test_server_connection_lost, 1);
+		test_server_connection_lost, 1,
+		NULL);
 	test_end();
 
 	test_begin("connection lost: manual retry");
@@ -787,7 +815,8 @@ static void test_connection_lost(void)
 	http_client_set.no_auto_retry = TRUE;
 	test_run_client_server(&http_client_set,
 		test_client_connection_lost,
-		test_server_connection_lost, 1);
+		test_server_connection_lost, 1,
+		NULL);
 	test_end();
 }
 
@@ -836,7 +865,7 @@ test_client_connection_lost_100_response(
 	}
 }
 
-static void
+static bool
 test_client_connection_lost_100(
 	const struct http_client_settings *client_set)
 {
@@ -869,7 +898,8 @@ test_client_connection_lost_100(
 	http_client_request_set_payload(hreq, input, TRUE);
 	http_client_request_submit(hreq);
 
-	i_stream_unref(&input);	
+	i_stream_unref(&input);
+	return TRUE;
 }
 
 /* test */
@@ -886,7 +916,8 @@ static void test_connection_lost_100(void)
 	http_client_set.max_attempts = 1;
 	test_run_client_server(&http_client_set,
 		test_client_connection_lost_100,
-		test_server_connection_lost_100, 1);
+		test_server_connection_lost_100, 1,
+		NULL);
 	test_end();
 }
 
@@ -964,7 +995,7 @@ test_client_connection_lost_sub_ioloop_response(
 	}
 }
 
-static void
+static bool
 test_client_connection_lost_sub_ioloop(
 	const struct http_client_settings *client_set)
 {
@@ -997,7 +1028,8 @@ test_client_connection_lost_sub_ioloop(
 	http_client_request_set_payload(hreq, input, TRUE);
 	http_client_request_submit(hreq);
 
-	i_stream_unref(&input);	
+	i_stream_unref(&input);
+	return TRUE;
 }
 
 /* test */
@@ -1014,7 +1046,8 @@ static void test_connection_lost_sub_ioloop(void)
 	http_client_set.max_attempts = 1;
 	test_run_client_server(&http_client_set,
 		test_client_connection_lost_sub_ioloop,
-		test_server_connection_lost_sub_ioloop, 2);
+		test_server_connection_lost_sub_ioloop, 2,
+		NULL);
 	test_end();
 }
 
@@ -1032,7 +1065,7 @@ test_early_success_input(struct server_connection *conn)
 		"Content-Length: 18\r\n"
 		"\r\n"
 		"Everything is OK\r\n";
-	
+
 	usleep(200000);
 	o_stream_nsend_str(conn->conn.output, resp);
 	server_connection_deinit(&conn);
@@ -1069,7 +1102,7 @@ test_client_early_success_response(
 	}
 }
 
-static void
+static bool
 test_client_early_success(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -1088,15 +1121,25 @@ test_client_early_success(const struct http_client_settings *client_set)
 	http_client_request_set_port(hreq, bind_ports[0]);
 
 	T_BEGIN {
-		payload = t_str_new(204800);
-		for (i = 0; i < 3200; i++) {
+		struct istream_chain *chain;
+		struct istream *input, *chain_input;
+
+		payload = t_str_new(64*3000);
+		for (i = 0; i < 3000; i++) {
 			str_append(payload,
 				"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n"
 				"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\r\n");
 		}
 
-		http_client_request_set_payload_data
-			(hreq, str_data(payload), str_len(payload));
+		chain_input = i_stream_create_chain(&chain);
+
+		input = i_stream_create_copy_from_data
+			(str_data(payload), str_len(payload));
+		i_stream_chain_append(chain, input);
+		i_stream_unref(&input);
+
+		http_client_request_set_payload(hreq, chain_input, FALSE);
+		i_stream_unref(&chain_input);
 	} T_END;
 	http_client_request_submit(hreq);
 
@@ -1105,6 +1148,8 @@ test_client_early_success(const struct http_client_settings *client_set)
 		test_client_early_success_response, ctx);
 	http_client_request_set_port(hreq, bind_ports[0]);
 	http_client_request_submit(hreq);
+
+	return TRUE;
 }
 
 /* test */
@@ -1119,7 +1164,8 @@ static void test_early_success(void)
 	test_begin("early succes");
 	test_run_client_server(&http_client_set,
 		test_client_early_success,
-		test_server_early_success, 1);
+		test_server_early_success, 1,
+		NULL);
 	test_end();
 }
 
@@ -1168,7 +1214,7 @@ test_client_bad_response_response(
 	}
 }
 
-static void
+static bool
 test_client_bad_response(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -1190,6 +1236,8 @@ test_client_bad_response(const struct http_client_settings *client_set)
 		test_client_bad_response_response, ctx);
 	http_client_request_set_port(hreq, bind_ports[0]);
 	http_client_request_submit(hreq);
+
+	return TRUE;
 }
 
 /* test */
@@ -1203,7 +1251,8 @@ static void test_bad_response(void)
 	test_begin("bad response");
 	test_run_client_server(&http_client_set,
 		test_client_bad_response,
-		test_server_bad_response, 1);
+		test_server_bad_response, 1,
+		NULL);
 	test_end();
 }
 
@@ -1248,7 +1297,7 @@ test_client_request_timed_out_response(
 	}
 }
 
-static void
+static bool
 test_client_request_timed_out(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -1270,6 +1319,8 @@ test_client_request_timed_out(const struct http_client_settings *client_set)
 		test_client_request_timed_out_response, ctx);
 	http_client_request_set_port(hreq, bind_ports[0]);
 	http_client_request_submit(hreq);
+
+	return TRUE;
 }
 
 /* test */
@@ -1285,7 +1336,8 @@ static void test_request_timed_out(void)
 	http_client_set.max_attempts = 1;
 	test_run_client_server(&http_client_set,
 		test_client_request_timed_out,
-		test_server_request_timed_out, 1);
+		test_server_request_timed_out, 1,
+		NULL);
 	test_end();
 
 	test_begin("request timed out: two attempts");
@@ -1293,7 +1345,8 @@ static void test_request_timed_out(void)
 	http_client_set.max_attempts = 1;
 	test_run_client_server(&http_client_set,
 		test_client_request_timed_out,
-		test_server_request_timed_out, 1);
+		test_server_request_timed_out, 1,
+		NULL);
 	test_end();
 
 	test_begin("request absolutely timed out");
@@ -1302,7 +1355,8 @@ static void test_request_timed_out(void)
 	http_client_set.max_attempts = 3;
 	test_run_client_server(&http_client_set,
 		test_client_request_timed_out,
-		test_server_request_timed_out, 1);
+		test_server_request_timed_out, 1,
+		NULL);
 	test_end();
 
 	test_begin("request double timed out");
@@ -1311,7 +1365,8 @@ static void test_request_timed_out(void)
 	http_client_set.max_attempts = 3;
 	test_run_client_server(&http_client_set,
 		test_client_request_timed_out,
-		test_server_request_timed_out, 1);
+		test_server_request_timed_out, 1,
+		NULL);
 	test_end();
 }
 
@@ -1383,7 +1438,7 @@ test_client_request_aborted_early_timeout(
 	}
 }
 
-static void
+static bool
 test_client_request_aborted_early(
 	const struct http_client_settings *client_set)
 {
@@ -1408,6 +1463,7 @@ test_client_request_aborted_early(
 
 	ctx->to = timeout_add_short(500,
 		test_client_request_aborted_early_timeout, ctx);
+	return TRUE;
 }
 
 /* test */
@@ -1421,7 +1477,99 @@ static void test_request_aborted_early(void)
 	test_begin("request aborted early");
 	test_run_client_server(&http_client_set,
 		test_client_request_aborted_early,
-		test_server_request_aborted_early, 1);
+		test_server_request_aborted_early, 1,
+		NULL);
+	test_end();
+}
+
+/*
+ * Request failed blocking
+ */
+
+/* server */
+
+static void
+test_request_failed_blocking_input(struct server_connection *conn)
+{
+	static const char *resp =
+		"HTTP/1.1 500 Internal Server Error\r\n"
+		"\r\n";
+
+	/* respond */
+	o_stream_nsend_str(conn->conn.output, resp);
+	sleep(10);
+	server_connection_deinit(&conn);
+}
+
+static void test_server_request_failed_blocking(unsigned int index)
+{
+	test_server_input = test_request_failed_blocking_input;
+	test_server_run(index);
+}
+
+/* client */
+
+struct _request_failed_blocking_ctx {
+	struct http_client_request *req;
+};
+
+static void
+test_client_request_failed_blocking_response(
+	const struct http_response *resp,
+	struct _request_failed_blocking_ctx *ctx ATTR_UNUSED)
+{
+	if (debug)
+		i_debug("RESPONSE: %u %s", resp->status, resp->reason);
+
+	i_assert(resp->status == 500);
+}
+
+static bool
+test_client_request_failed_blocking(
+	const struct http_client_settings *client_set)
+{
+	static const char *payload = "This a test payload!";
+	struct http_client_request *hreq;
+	struct _request_failed_blocking_ctx *ctx;
+	unsigned int n;
+	string_t *data;
+
+	data = str_new(default_pool, 1000000);
+	for (n = 0; n < 50000; n++)
+		str_append(data, payload);
+
+	ctx = i_new(struct _request_failed_blocking_ctx, 1);
+
+	http_client = http_client_init(client_set);
+
+	hreq = ctx->req = http_client_request(http_client,
+		"GET", net_ip2addr(&bind_ip), "/request-failed-blocking.txt",
+		test_client_request_failed_blocking_response, ctx);
+	http_client_request_set_port(hreq, bind_ports[0]);
+
+	test_assert(http_client_request_send_payload(&hreq,
+		str_data(data), str_len(data)) < 0);
+	i_assert(hreq == NULL);
+
+	str_free(&data);
+	i_free(ctx);
+	return FALSE;
+}
+
+/* test */
+
+static void test_request_failed_blocking(void)
+{
+	struct http_client_settings http_client_set;
+
+	test_client_defaults(&http_client_set);
+	http_client_set.socket_send_buffer_size = 4096;
+
+	test_begin("request failed blocking");
+	test_run_client_server(&http_client_set,
+		test_client_request_failed_blocking,
+		test_server_request_failed_blocking, 1,
+		NULL);
 	test_end();
 }
 
@@ -1484,7 +1632,7 @@ test_client_client_deinit_early_timeout(
 	io_loop_stop(ioloop);
 }
 
-static void
+static bool
 test_client_client_deinit_early(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -1508,6 +1656,7 @@ test_client_client_deinit_early(const struct http_client_settings *client_set)
 
 	ctx->to = timeout_add_short(500,
 		test_client_client_deinit_early_timeout, ctx);
+	return TRUE;
 }
 
 /* test */
@@ -1521,7 +1670,8 @@ static void test_client_deinit_early(void)
 	test_begin("client deinit early");
 	test_run_client_server(&http_client_set,
 		test_client_client_deinit_early,
-		test_server_client_deinit_early, 1);
+		test_server_client_deinit_early, 1,
+		NULL);
 	test_end();
 }
 
@@ -1576,7 +1726,7 @@ test_client_retry_with_delay_response(
 		/* check delay */
 		real_delay = timeval_diff_msecs(&ioloop_timeval, &ctx->time);
 		exp_delay = (1 << (ctx->retries-1)) * 50;
-		if (real_delay < exp_delay) {
+		if (real_delay < exp_delay-2) {
 			i_fatal("Retry delay is too short %d < %d",
 				real_delay, exp_delay);
 		}
@@ -1595,7 +1745,7 @@ test_client_retry_with_delay_response(
 	io_loop_stop(ioloop);
 }
 
-static void
+static bool
 test_client_retry_with_delay(const struct http_client_settings *client_set)
 {
 	struct http_client_request *hreq;
@@ -1611,6 +1761,8 @@ test_client_retry_with_delay(const struct http_client_settings *client_set)
 		test_client_retry_with_delay_response, ctx);
 	http_client_request_set_port(hreq, bind_ports[0]);
 	http_client_request_submit(hreq);
+
+	return TRUE;
 }
 
 /* test */
@@ -1625,10 +1777,553 @@ static void test_retry_with_delay(void)
 	test_begin("retry with delay");
 	test_run_client_server(&http_client_set,
 		test_client_retry_with_delay,
-		test_server_retry_with_delay, 1);
+		test_server_retry_with_delay, 1,
+		NULL);
 	test_end();
 }
 
+/*
+ * DNS service failure
+ */
+
+/* client */
+
+struct _dns_service_failure {
+	unsigned int count;
+};
+
+static void
+test_client_dns_service_failure_response(
+	const struct http_response *resp,
+	struct _dns_service_failure *ctx)
+{
+	if (debug)
+		i_debug("RESPONSE: %u %s", resp->status, resp->reason);
+
+	test_assert(resp->status == HTTP_CLIENT_REQUEST_ERROR_HOST_LOOKUP_FAILED);
+	test_assert(resp->reason != NULL && *resp->reason != '\0');
+
+	if (--ctx->count == 0) {
+		i_free(ctx);
+		io_loop_stop(ioloop);
+	}
+}
+
+static bool
+test_client_dns_service_failure(const struct http_client_settings *client_set)
+{
+	struct http_client_request *hreq;
+	struct _dns_service_failure *ctx;
+
+	ctx = i_new(struct _dns_service_failure, 1);
+	ctx->count = 2;
+
+	http_client = http_client_init(client_set);
+
+	hreq = http_client_request(http_client,
+		"GET", "example.com", "/dns-service-failure.txt",
+		test_client_dns_service_failure_response, ctx);
+	http_client_request_set_port(hreq, 80);
+	http_client_request_submit(hreq);
+
+	hreq = http_client_request(http_client,
+		"GET", "example.com", "/dns-service-failure2.txt",
+		test_client_dns_service_failure_response, ctx);
+	http_client_request_set_port(hreq, 80);
+	http_client_request_submit(hreq);
+
+	return TRUE;
+}
+
+/* test */
+
+static void test_dns_service_failure(void)
+{
+	struct http_client_settings http_client_set;
+
+	test_client_defaults(&http_client_set);
+	http_client_set.dns_client_socket_path = "./frop";
+
+	test_begin("dns service failure");
+	test_run_client_server(&http_client_set,
+		test_client_dns_service_failure,
+		NULL, 0, NULL);
+	test_end();
+}
+
+/*
+ * DNS timeout
+ */
+
+/* dns */
+
+static void
+test_dns_timeout_input(struct server_connection *conn ATTR_UNUSED)
+{
+	/* hang */
+	sleep(100);
+	server_connection_deinit(&conn);
+}
+
+static void test_dns_dns_timeout(void)
+{
+	test_server_input = test_dns_timeout_input;
+	test_server_run(0);
+}
+
+/* client */
+
+struct _dns_timeout {
+	unsigned int count;
+};
+
+static void
+test_client_dns_timeout_response(
+	const struct http_response *resp,
+	struct _dns_timeout *ctx)
+{
+	if (debug)
+		i_debug("RESPONSE: %u %s", resp->status, resp->reason);
+
+	test_assert(resp->status == HTTP_CLIENT_REQUEST_ERROR_HOST_LOOKUP_FAILED);
+	test_assert(resp->reason != NULL && *resp->reason != '\0');
+
+	if (--ctx->count == 0) {
+		i_free(ctx);
+		io_loop_stop(ioloop);
+	}
+}
+
+static bool
+test_client_dns_timeout(const struct http_client_settings *client_set)
+{
+	struct http_client_request *hreq;
+	struct _dns_timeout *ctx;
+
+	ctx = i_new(struct _dns_timeout, 1);
+	ctx->count = 2;
+
+	http_client = http_client_init(client_set);
+
+	hreq = http_client_request(http_client,
+		"GET", "example.com", "/dns-timeout.txt",
+		test_client_dns_timeout_response, ctx);
+	http_client_request_set_port(hreq, 80);
+	http_client_request_submit(hreq);
+
+	hreq = http_client_request(http_client,
+		"GET", "example.com", "/dns-timeout2.txt",
+		test_client_dns_timeout_response, ctx);
+	http_client_request_set_port(hreq, 80);
+	http_client_request_submit(hreq);
+
+	return TRUE;
+}
+
+/* test */
+
+static void test_dns_timeout(void)
+{
+	struct http_client_settings http_client_set;
+
+	test_client_defaults(&http_client_set);
+	http_client_set.request_timeout_msecs = 2000;
+	http_client_set.connect_timeout_msecs = 2000;
+	http_client_set.dns_client_socket_path = "./dns-test";
+
+	test_begin("dns timeout");
+	test_run_client_server(&http_client_set,
+		test_client_dns_timeout,
+		NULL, 0,
+		test_dns_dns_timeout);
+	test_end();
+}
+
+/*
+ * DNS lookup failure
+ */
+
+/* dns */
+
+static void
+test_dns_lookup_failure_input(struct server_connection *conn)
+{
+	o_stream_nsend_str(conn->conn.output,
+		t_strdup_printf("%d\n", EAI_FAIL));
+	server_connection_deinit(&conn);
+}
+
+static void test_dns_dns_lookup_failure(void)
+{
+	test_server_input = test_dns_lookup_failure_input;
+	test_server_run(0);
+}
+
+/* client */
+
+struct _dns_lookup_failure {
+	unsigned int count;
+};
+
+static void
+test_client_dns_lookup_failure_response(
+	const struct http_response *resp,
+	struct _dns_lookup_failure *ctx)
+{
+	if (debug)
+		i_debug("RESPONSE: %u %s", resp->status, resp->reason);
+
+	test_assert(resp->status == HTTP_CLIENT_REQUEST_ERROR_HOST_LOOKUP_FAILED);
+	test_assert(resp->reason != NULL && *resp->reason != '\0');
+
+	if (--ctx->count == 0) {
+		i_free(ctx);
+		io_loop_stop(ioloop);
+	}
+}
+
+static bool
+test_client_dns_lookup_failure(const struct http_client_settings *client_set)
+{
+	struct http_client_request *hreq;
+	struct _dns_lookup_failure *ctx;
+
+	ctx = i_new(struct _dns_lookup_failure, 1);
+	ctx->count = 2;
+
+	http_client = http_client_init(client_set);
+
+	hreq = http_client_request(http_client,
+		"GET", "example.com", "/dns-lookup-failure.txt",
+		test_client_dns_lookup_failure_response, ctx);
+	http_client_request_set_port(hreq, 80);
+	http_client_request_submit(hreq);
+
+	hreq = http_client_request(http_client,
+		"GET", "example.com", "/dns-lookup-failure2.txt",
+		test_client_dns_lookup_failure_response, ctx);
+	http_client_request_set_port(hreq, 80);
+	http_client_request_submit(hreq);
+
+	return TRUE;
+}
+
+/* test */
+
+static void test_dns_lookup_failure(void)
+{
+	struct http_client_settings http_client_set;
+
+	test_client_defaults(&http_client_set);
+	http_client_set.dns_client_socket_path = "./dns-test";
+
+	test_begin("dns lookup failure");
+	test_run_client_server(&http_client_set,
+		test_client_dns_lookup_failure,
+		NULL, 0,
+		test_dns_dns_lookup_failure);
+	test_end();
+}
+
+/*
+ * DNS lookup ttl
+ */
+
+/* dns */
+
+static void
+test_dns_lookup_ttl_input(struct server_connection *conn)
+{
+	static unsigned int count = 0;
+	const char *line;
+
+	while ((line=i_stream_read_next_line(conn->conn.input)) != NULL) {
+		if (debug)
+			i_debug("DNS REQUEST %u: %s", count, line);
+
+		if (count == 0) {
+			o_stream_nsend_str(conn->conn.output,
+				"0 1\n127.0.0.1\n");
+		} else {
+			o_stream_nsend_str(conn->conn.output,
+				t_strdup_printf("%d\n", EAI_FAIL));
+			if (count > 4) {
+				server_connection_deinit(&conn);
+				return;
+			}
+		}
+		count++;
+	}
+}
+
+static void test_dns_dns_lookup_ttl(void)
+{
+	test_server_input = test_dns_lookup_ttl_input;
+	test_server_run(0);
+}
+
+/* server */
+
+static void
+test_server_dns_lookup_ttl_input(struct server_connection *conn)
+{
+	string_t *resp;
+
+	resp = t_str_new(512);
+	str_printfa(resp,
+		"HTTP/1.1 200 OK\r\n"
+		"Connection: close\r\n"
+		"\r\n");
+	o_stream_nsend(conn->conn.output,
+		str_data(resp), str_len(resp));
+	server_connection_deinit(&conn);
+}
+
+static void test_server_dns_lookup_ttl(unsigned int index)
+{
+	test_server_input = test_server_dns_lookup_ttl_input;
+	test_server_run(index);
+}
+
+/* client */
+
+struct _dns_lookup_ttl {
+	struct http_client *client;
+	unsigned int count;
+	struct timeout *to;
+};
+
+static void
+test_client_dns_lookup_ttl_response_stage2(
+	const struct http_response *resp,
+	struct _dns_lookup_ttl *ctx)
+{
+	if (debug)
+		i_debug("RESPONSE: %u %s", resp->status, resp->reason);
+
+	test_assert(resp->status == HTTP_CLIENT_REQUEST_ERROR_HOST_LOOKUP_FAILED);
+	test_assert(resp->reason != NULL && *resp->reason != '\0');
+
+	if (--ctx->count == 0) {
+		i_free(ctx);
+		io_loop_stop(ioloop);
+	}
+}
+
+static void
+test_client_dns_lookup_ttl_stage2_start(struct _dns_lookup_ttl *ctx)
+{
+	struct http_client_request *hreq;
+
+	timeout_remove(&ctx->to);
+
+	ctx->count = 2;
+
+	hreq = http_client_request(ctx->client,
+		"GET", "example.com", "/dns-lookup-ttl-stage2.txt",
+		test_client_dns_lookup_ttl_response_stage2, ctx);
+	http_client_request_set_port(hreq, bind_ports[0]);
+	http_client_request_submit(hreq);
+
+	hreq = http_client_request(ctx->client,
+		"GET", "example.com", "/dns-lookup-ttl2-stage2.txt",
+		test_client_dns_lookup_ttl_response_stage2, ctx);
+	http_client_request_set_port(hreq, bind_ports[0]);
+	http_client_request_submit(hreq);
+}
+
+static void
+test_client_dns_lookup_ttl_response_stage1(
+	const struct http_response *resp,
+	struct _dns_lookup_ttl *ctx)
+{
+	if (debug)
+		i_debug("RESPONSE: %u %s", resp->status, resp->reason);
+
+	test_assert(resp->status == 200);
+
+	if (--ctx->count == 0) {
+		ctx->to = timeout_add(2000,
+			test_client_dns_lookup_ttl_stage2_start, ctx);
+	}
+}
+
+static bool
+test_client_dns_lookup_ttl(const struct http_client_settings *client_set)
+{
+	struct http_client_request *hreq;
+	struct _dns_lookup_ttl *ctx;
+
+	ctx = i_new(struct _dns_lookup_ttl, 1);
+	ctx->count = 2;
+
+	ctx->client = http_client = http_client_init(client_set);
+
+	hreq = http_client_request(ctx->client,
+		"GET", "example.com", "/dns-lookup-ttl-stage1.txt",
+		test_client_dns_lookup_ttl_response_stage1, ctx);
+	http_client_request_set_port(hreq, bind_ports[0]);
+	http_client_request_submit(hreq);
+
+	hreq = http_client_request(ctx->client,
+		"GET", "example.com", "/dns-lookup-ttl2-stage1.txt",
+		test_client_dns_lookup_ttl_response_stage1, ctx);
+	http_client_request_set_port(hreq, bind_ports[0]);
+	http_client_request_submit(hreq);
+
+	return TRUE;
+}
+
+/* test */
+
+static void test_dns_lookup_ttl(void)
+{
+	struct http_client_settings http_client_set;
+
+	test_client_defaults(&http_client_set);
+	http_client_set.dns_client_socket_path = "./dns-test";
+	http_client_set.dns_ttl_msecs = 1000;
+
+	test_begin("dns lookup ttl");
+	test_run_client_server(&http_client_set,
+		test_client_dns_lookup_ttl,
+		test_server_dns_lookup_ttl, 1,
+		test_dns_dns_lookup_ttl);
+	test_end();
+}
+
+/*
+ * Peer reuse failure
+ */
+
+/* server */
+
+static void
+test_peer_reuse_failure_input(struct server_connection *conn)
+{
+	static unsigned int seq = 0;
+	static const char *resp =
+		"HTTP/1.1 200 OK\r\n"
+		"\r\n";
+	o_stream_nsend_str(conn->conn.output, resp);
+	if (seq++ > 2) {
+		server_connection_deinit(&conn);
+		io_loop_stop(current_ioloop);
+	}
+}
+
+static void test_server_peer_reuse_failure(unsigned int index)
+{
+	test_server_input = test_peer_reuse_failure_input;
+	test_server_run(index);
+}
+
+/* client */
+
+struct _peer_reuse_failure {
+	struct timeout *to;
+	bool first:1;
+};
+
+static void
+test_client_peer_reuse_failure_response2(
+	const struct http_response *resp,
+	struct _peer_reuse_failure *ctx)
+{
+	if (debug)
+		i_debug("RESPONSE: %u %s", resp->status, resp->reason);
+
+	test_assert(resp->status == HTTP_CLIENT_REQUEST_ERROR_CONNECT_FAILED);
+	test_assert(resp->reason != NULL && *resp->reason != '\0');
+	i_free(ctx);
+	io_loop_stop(ioloop);
+}
+
+static void
+test_client_peer_reuse_failure_next(struct _peer_reuse_failure *ctx)
+{
+	struct http_client_request *hreq;
+
+	timeout_remove(&ctx->to);
+
+	hreq = http_client_request(http_client,
+		"GET", net_ip2addr(&bind_ip), "/peer-reuse-next.txt",
+		test_client_peer_reuse_failure_response2, ctx);
+	http_client_request_set_port(hreq, bind_ports[0]);
+	http_client_request_submit(hreq);
+}
+
+static void
+test_client_peer_reuse_failure_response1(
+	const struct http_response *resp,
+	struct _peer_reuse_failure *ctx)
+{
+	if (debug)
+		i_debug("RESPONSE: %u %s", resp->status, resp->reason);
+
+	if (ctx->first) {
+		test_assert(resp->status == 200);
+
+		ctx->first = FALSE;
+		ctx->to = timeout_add_short(500, test_client_peer_reuse_failure_next, ctx);
+	} else {
+		test_assert(resp->status == HTTP_CLIENT_REQUEST_ERROR_CONNECT_FAILED);
+	}
+
+	test_assert(resp->reason != NULL && *resp->reason != '\0');
+}
+
+static bool
+test_client_peer_reuse_failure(const struct http_client_settings *client_set)
+{
+	struct http_client_request *hreq;
+	struct _peer_reuse_failure *ctx;
+
+	ctx = i_new(struct _peer_reuse_failure, 1);
+	ctx->first = TRUE;
+
+	http_client = http_client_init(client_set);
+
+	hreq = http_client_request(http_client,
+		"GET", net_ip2addr(&bind_ip), "/peer-reuse.txt",
+		test_client_peer_reuse_failure_response1, ctx);
+	http_client_request_set_port(hreq, bind_ports[0]);
+	http_client_request_submit(hreq);
+
+	hreq = http_client_request(http_client,
+		"GET", net_ip2addr(&bind_ip), "/peer-reuse.txt",
+		test_client_peer_reuse_failure_response1, ctx);
+	http_client_request_set_port(hreq, bind_ports[0]);
+	http_client_request_submit(hreq);
+
+	hreq = http_client_request(http_client,
+		"GET", net_ip2addr(&bind_ip), "/peer-reuse.txt",
+		test_client_peer_reuse_failure_response1, ctx);
+	http_client_request_set_port(hreq, bind_ports[0]);
+	http_client_request_submit(hreq);
+
+	return TRUE;
+}
+
+/* test */
+
+static void test_peer_reuse_failure(void)
+{
+	struct http_client_settings http_client_set;
+
+	test_client_defaults(&http_client_set);
+	http_client_set.max_connect_attempts = 1;
+	http_client_set.max_idle_time_msecs = 500;
+
+	test_begin("peer reuse failure");
+	test_run_client_server(&http_client_set,
+		test_client_peer_reuse_failure,
+		test_server_peer_reuse_failure, 1,
+		NULL);
+	test_end();
+}
+
+
 /*
  * All tests
  */
@@ -1648,8 +2343,14 @@ static void (*test_functions[])(void) = {
 	test_bad_response,
 	test_request_timed_out,
 	test_request_aborted_early,
+	test_request_failed_blocking,
 	test_client_deinit_early,
 	test_retry_with_delay,
+	test_dns_service_failure,
+	test_dns_timeout,
+	test_dns_lookup_failure,
+	test_dns_lookup_ttl,
+	test_peer_reuse_failure,
 	NULL
 };
 
@@ -1805,13 +2506,20 @@ static void test_servers_kill_all(void)
 		}
 	}
 	server_pids_count = 0;
+
+	if (dns_pid != (pid_t)-1) {
+		(void)kill(dns_pid, SIGKILL);
+		(void)waitpid(dns_pid, NULL, 0);
+		dns_pid = (pid_t)-1;
+	}
 }
 
 static void test_run_client_server(
 	const struct http_client_settings *client_set,
 	test_client_init_t client_test,
 	test_server_init_t server_test,
-	unsigned int server_tests_count)
+	unsigned int server_tests_count,
+	test_dns_init_t dns_test)
 {
 	unsigned int i;
 
@@ -1861,19 +2569,53 @@ static void test_run_client_server(
 			i_debug("client: PID=%s", my_pid);
 	}
 
+	if (dns_test != NULL) {
+		int fd;
+
+		i_unlink_if_exists("./dns-test");
+		fd = net_listen_unix("./dns-test", 128);
+		if (fd == -1) {
+			i_fatal("listen(./dns-test) failed: %m");
+		}
+
+		fd_listen = fd;
+		if ((dns_pid = fork()) == (pid_t)-1)
+			i_fatal("fork() failed: %m");
+		if (dns_pid == 0) {
+			dns_pid = (pid_t)-1;
+			hostpid_init();
+			if (debug)
+				i_debug("dns server: PID=%s", my_pid);
+			/* child: server */
+			ioloop = io_loop_create();
+			dns_test();
+			io_loop_destroy(&ioloop);
+			if (fd_listen != -1)
+				i_close_fd(&fd_listen);
+			/* wait for it to be killed; this way, valgrind will not
+			   object to this process going away inelegantly. */
+			sleep(60);
+			exit(1);
+		}
+		if (fd_listen != -1)
+			i_close_fd(&fd_listen);
+	}
+
 	/* parent: client */
 
 	usleep(100000); /* wait a little for server setup */
 
 	ioloop = io_loop_create();
-	client_test(client_set);
-	io_loop_run(ioloop);
+	if (client_test(client_set))
+		io_loop_run(ioloop);
 	test_client_deinit();
 	io_loop_destroy(&ioloop);
 
 	test_servers_kill_all();
 	i_free(server_pids);
 	i_free(bind_ports);
+
+	i_unlink_if_exists("./dns-test");
 }
 
 /*
