@@ -1,6 +1,8 @@
 #ifndef IMAPC_CLIENT_H
 #define IMAPC_CLIENT_H
 
+#include "net.h"
+
 /* IMAP RFC defines this to be at least 30 minutes. */
 #define IMAPC_DEFAULT_MAX_IDLE_TIME (60*29)
 
@@ -54,6 +56,7 @@ enum imapc_client_ssl_mode {
 
 #define IMAPC_DEFAULT_CONNECT_TIMEOUT_MSECS (1000*30)
 #define IMAPC_DEFAULT_COMMAND_TIMEOUT_MSECS (1000*60*5)
+#define IMAPC_DEFAULT_MAX_LINE_LENGTH ((size_t)-1)
 
 struct imapc_throttling_settings {
 	unsigned int init_msecs;
@@ -63,7 +66,7 @@ struct imapc_throttling_settings {
 
 struct imapc_client_settings {
 	const char *host;
-	unsigned int port;
+	in_port_t port;
 
 	const char *master_user;
 	const char *username;
@@ -71,6 +74,7 @@ struct imapc_client_settings {
 	/* Space-separated list of SASL mechanisms to try (in the specified
 	   order). The default is to use only LOGIN command or SASL PLAIN. */
 	const char *sasl_mechanisms;
+	bool use_proxyauth; /* Use Sun/Oracle PROXYAUTH command */
 	unsigned int max_idle_time;
 
 	const char *dns_client_socket_path;
@@ -90,6 +94,10 @@ struct imapc_client_settings {
 	   sent or received. 0 = default. */
 	unsigned int cmd_timeout_msecs;
 
+	/* Maximum allowed line length (not including literals read as
+	   streams). 0 = unlimited. */
+	size_t max_line_length;
+
 	struct imapc_throttling_settings throttle_set;
 };
 
