@@ -36,13 +36,13 @@ static void parse_content_type(struct message_part_body_data *data,
 	string_t *str;
 	unsigned int i;
 	bool charset_found = FALSE;
+	int ret;
 
 	rfc822_parser_init(&parser, hdr->full_value, hdr->full_value_len, NULL);
 	rfc822_skip_lwsp(&parser);
 
 	str = t_str_new(256);
-	if (rfc822_parse_content_type(&parser, str) < 0)
-		return;
+	ret = rfc822_parse_content_type(&parser, str);
 
 	/* Save content type and subtype */
 	value = str_c(str);
@@ -56,6 +56,16 @@ static void parse_content_type(struct message_part_body_data *data,
 	str_truncate(str, i);
 	data->content_type = imap_get_string(data->pool, str_c(str));
 
+	if (ret < 0) {
+		/* Content-Type is broken, but we wanted to get it as well as
+		   we could. Don't try to read the parameters anymore though.
+
+		   We don't completely ignore a broken Content-Type, because
+		   then it would be written as text/plain. This would cause a
+		   mismatch with the message_part's MESSAGE_PART_FLAG_TEXT. */
+		return;
+	}
+
 	/* parse parameters and save them */
 	str_truncate(str, 0);
 	rfc2231_parse(&parser, &results);
@@ -730,10 +740,16 @@ imap_bodystructure_parse_args(const struct imap_arg *args, pool_t pool,
 	text = strcasecmp(content_type, "text") == 0;
 	message_rfc822 = strcasecmp(content_type, "message") == 0 &&
 		strcasecmp(subtype, "rfc822") == 0;
+#if 0
+	/* Disabled for now. Earlier Dovecot versions handled broken
+	   Content-Type headers by writing them as "text" "plain" to
+	   BODYSTRUCTURE reply, but the message_part didn't have
+	   MESSAGE_PART_FLAG_TEXT. */
 	if (text != ((part->flags & MESSAGE_PART_FLAG_TEXT) != 0)) {
 		*error_r = "message_part text flag doesn't match BODYSTRUCTURE";
 		return -1;
 	}
+#endif
 	if (message_rfc822 != ((part->flags & MESSAGE_PART_FLAG_MESSAGE_RFC822) != 0)) {
 		*error_r = "message_part message/rfc822 flag doesn't match BODYSTRUCTURE";
 		return -1;
