@@ -289,6 +289,14 @@ mail_cache_transaction_lookup_rec(struct mail_cache_transaction_ctx *ctx,
 	const struct mail_cache_transaction_rec *recs;
 	unsigned int i, count;
 
+	if (MAIL_CACHE_IS_UNUSABLE(ctx->cache) ||
+	    ctx->cache_file_seq != ctx->cache->hdr->file_seq) {
+		/* Cache was compressed during this transaction. We can't
+		   safely use the data anymore, since its fields won't match
+		   cache->file_fields_map. */
+		return NULL;
+	}
+
 	recs = array_get(&ctx->cache_data_seq, &count);
 	for (i = *trans_next_idx; i < count; i++) {
 		if (recs[i].seq == seq) {
