@@ -10,6 +10,7 @@
 #include "read-full.h"
 #include "write-full.h"
 #include "mail-cache-private.h"
+#include "ioloop.h"
 
 #include <unistd.h>
 
@@ -480,7 +481,11 @@ int mail_cache_map(struct mail_cache *cache, size_t offset, size_t size,
 	if (cache->mmap_base == MAP_FAILED) {
 		cache->mmap_base = NULL;
 		cache->mmap_length = 0;
-		mail_cache_set_syscall_error(cache, "mmap()");
+		if (ioloop_time != cache->last_mmap_error_time) {
+			cache->last_mmap_error_time = ioloop_time;
+			mail_cache_set_syscall_error(cache, t_strdup_printf(
+				"mmap(size=%"PRIuSIZE_T")", cache->mmap_length));
+		}
 		return -1;
 	}
 	*data_r = offset > cache->mmap_length ? NULL :
