@@ -67,6 +67,8 @@ static int mail_index_recreate(struct mail_index *index)
 
 	i_assert(!MAIL_INDEX_IS_IN_MEMORY(index));
 	i_assert(map->hdr.indexid == index->indexid);
+	i_assert((map->hdr.flags & MAIL_INDEX_HDR_FLAG_CORRUPTED) == 0);
+	i_assert(index->indexid != 0);
 
 	fd = mail_index_create_tmp_file(index, index->filepath, &path);
 	if (fd == -1)
