@@ -16,7 +16,8 @@
 #include "mail-index-view-private.h"
 #include "mail-index-sync-private.h"
 #include "mail-index-modseq.h"
-#include "mail-transaction-log.h"
+#include "mail-transaction-log-private.h"
+#include "mail-transaction-log-view-private.h"
 #include "mail-cache.h"
 
 #include <stdio.h>
@@ -49,6 +50,15 @@ struct mail_index *mail_index_alloc(const char *dir, const char *prefix)
 	index->lock_method = FILE_LOCK_METHOD_FCNTL;
 	index->max_lock_timeout_secs = UINT_MAX;
 
+	index->log_rotate_min_size =
+		MAIL_TRANSACTION_LOG_ROTATE_DEFAULT_MIN_SIZE;
+	index->log_rotate_max_size =
+		MAIL_TRANSACTION_LOG_ROTATE_DEFAULT_MAX_SIZE;
+	index->log_rotate_min_created_ago_secs =
+		MAIL_TRANSACTION_LOG_ROTATE_DEFAULT_TIME;
+	index->log_rotate_log2_stale_secs =
+		MAIL_TRANSACTION_LOG2_DEFAULT_STALE_SECS;
+
 	index->keywords_ext_id =
 		mail_index_ext_register(index, MAIL_INDEX_EXT_KEYWORDS,
 					128, 2, 1);
@@ -112,6 +122,17 @@ void mail_index_set_lock_method(struct mail_index *index,
 	index->max_lock_timeout_secs = max_timeout_secs;
 }
 
+void mail_index_set_log_rotation(struct mail_index *index,
+				 uoff_t min_size, uoff_t max_size,
+				 unsigned int min_created_ago_secs,
+				 unsigned int log2_stale_secs)
+{
+	index->log_rotate_min_size = min_size;
+	index->log_rotate_max_size = max_size;
+	index->log_rotate_min_created_ago_secs = min_created_ago_secs;
+	index->log_rotate_log2_stale_secs = log2_stale_secs;
+}
+
 void mail_index_set_ext_init_data(struct mail_index *index, uint32_t ext_id,
 				  const void *data, size_t size)
 {
@@ -600,8 +621,16 @@ int mail_index_open(struct mail_index *index, enum mail_index_open_flags flags)
 		mail_index_close_nonopened(index);
 		return ret;
 	}
+
 	index->open_count++;
 
+	if (index->log->head == NULL) {
+		mail_index_close(index);
+		mail_index_set_error(index, "Index is corrupted "
+					    "(log->view->head == NULL)");
+		return -1;
+	}
+
 	i_assert(index->map != NULL);
 	mail_index_alloc_cache_index_opened(index);
 	return 1;
@@ -689,21 +718,27 @@ int mail_index_unlink(struct mail_index *index)
 	}
 }
 
-int mail_index_reopen_if_changed(struct mail_index *index)
+int mail_index_reopen_if_changed(struct mail_index *index,
+				 const char **reason_r)
 {
 	struct stat st1, st2;
+	int ret;
 
-	if (MAIL_INDEX_IS_IN_MEMORY(index))
+	if (MAIL_INDEX_IS_IN_MEMORY(index)) {
+		*reason_r = "in-memory index";
 		return 0;
+	}
 
 	if (index->fd == -1)
-		return mail_index_try_open_only(index);
+		goto final;
 
 	if ((index->flags & MAIL_INDEX_OPEN_FLAG_NFS_FLUSH) != 0)
 		nfs_flush_file_handle_cache(index->filepath);
 	if (nfs_safe_stat(index->filepath, &st2) < 0) {
-		if (errno == ENOENT)
+		if (errno == ENOENT) {
+			*reason_r = "index not found via stat()";
 			return 0;
+		}
 		mail_index_set_syscall_error(index, "stat()");
 		return -1;
 	}
@@ -714,17 +749,26 @@ int mail_index_reopen_if_changed(struct mail_index *index)
 			return -1;
 		}
 		/* deleted/recreated, reopen */
+		*reason_r = "index is stale";
 	} else if (st1.st_ino == st2.st_ino &&
 		   CMP_DEV_T(st1.st_dev, st2.st_dev)) {
 		/* the same file */
+		*reason_r = "index unchanged";
 		return 1;
+	} else {
+		*reason_r = "index inode changed";
 	}
 
 	/* new file, new locks. the old fd can keep its locks, they don't
 	   matter anymore as no-one's going to modify the file. */
 	mail_index_close_file(index);
 
-	return mail_index_try_open_only(index);
+final:
+	if ((ret = mail_index_try_open_only(index)) == 0)
+		*reason_r = "index not found via open()";
+	else if (ret > 0)
+		*reason_r = "index opened";
+	return ret;
 }
 
 int mail_index_refresh(struct mail_index *index)
@@ -753,10 +797,7 @@ void mail_index_set_error(struct mail_index *index, const char *fmt, ...)
 		index->error = i_strdup_vprintf(fmt, va);
 		va_end(va);
 
-		if (ioloop_time != index->last_error_time) {
-			index->last_error_time = ioloop_time;
-			i_error("%s", index->error);
-		}
+		i_error("%s", index->error);
 	}
 }
 
