@@ -34,7 +34,9 @@ enum mail_index_open_flags {
 	MAIL_INDEX_OPEN_FLAG_NEVER_IN_MEMORY	= 0x200,
 	/* We're only going to save new messages to the index.
 	   Avoid unnecessary reads. */
-	MAIL_INDEX_OPEN_FLAG_SAVEONLY		= 0x400
+	MAIL_INDEX_OPEN_FLAG_SAVEONLY		= 0x400,
+	/* Enable debug logging */
+	MAIL_INDEX_OPEN_FLAG_DEBUG		= 0x800,
 };
 
 enum mail_index_header_compat_flags {
@@ -44,7 +46,10 @@ enum mail_index_header_compat_flags {
 enum mail_index_header_flag {
 	/* Index file is corrupted, reopen or recreate it. */
 	MAIL_INDEX_HDR_FLAG_CORRUPTED		= 0x0001,
-	MAIL_INDEX_HDR_FLAG_HAVE_DIRTY		= 0x0002
+	MAIL_INDEX_HDR_FLAG_HAVE_DIRTY		= 0x0002,
+	/* Index has been fsck'd. The caller may want to resync the index
+	   to make sure it's valid and drop this flag. */
+	MAIL_INDEX_HDR_FLAG_FSCKD		= 0x0004,
 };
 
 enum mail_index_mail_flags {
@@ -103,6 +108,7 @@ struct mail_index_header {
 	uint32_t day_first_uid[8];
 };
 
+#define MAIL_INDEX_RECORD_MIN_SIZE (sizeof(uint32_t) + sizeof(uint8_t))
 struct mail_index_record {
 	uint32_t uid;
 	uint8_t flags; /* enum mail_flags | enum mail_index_mail_flags */
@@ -167,7 +173,10 @@ enum mail_index_sync_flags {
 	/* Same as MAIL_INDEX_SYNC_FLAG_DELETING_INDEX, but finish index
 	   deletion only once and fail the rest (= avoid race conditions when
 	   multiple processes try to mark the index deleted) */
-	MAIL_INDEX_SYNC_FLAG_TRY_DELETING_INDEX	= 0x40
+	MAIL_INDEX_SYNC_FLAG_TRY_DELETING_INDEX	= 0x40,
+	/* Update header's tail_offset to head_offset, even if it's the only
+	   thing we do and there's no strict need for it. */
+	MAIL_INDEX_SYNC_FLAG_UPDATE_TAIL_OFFSET	= 0x80
 };
 
 enum mail_index_view_sync_flags {
@@ -240,6 +249,14 @@ void mail_index_set_permissions(struct mail_index *index,
 void mail_index_set_lock_method(struct mail_index *index,
 				enum file_lock_method lock_method,
 				unsigned int max_timeout_secs);
+/* Rotate transaction log after it's a) min_size or larger and it was created
+   at least min_created_ago_secs or b) larger than max_size. Delete .log.2 when
+   it's older than log2_stale_secs. The defaults are min_size=32kB, max_size=1M,
+   min_created_ago_secs=5min, log2_stale_secs=2d. */
+void mail_index_set_log_rotation(struct mail_index *index,
+				 uoff_t min_size, uoff_t max_size,
+				 unsigned int min_created_ago_secs,
+				 unsigned int log2_stale_secs);
 /* When creating a new index file or reseting an existing one, add the given
    extension header data immediately to it. */
 void mail_index_set_ext_init_data(struct mail_index *index, uint32_t ext_id,
@@ -303,6 +320,11 @@ void mail_index_transaction_reset(struct mail_index_transaction *t);
 void mail_index_transaction_set_max_modseq(struct mail_index_transaction *t,
 					   uint64_t max_modseq,
 					   ARRAY_TYPE(seq_range) *seqs);
+/* Returns the resulting highest-modseq after this commit. This can be called
+   only if transaction log is locked, which normally means only during mail
+   index syncing. If there are any appends, they all must have been assigned
+   UIDs before calling this. */
+uint64_t mail_index_transaction_get_highest_modseq(struct mail_index_transaction *t);
 
 /* Returns the view transaction was created for. */
 struct mail_index_view *
@@ -360,6 +382,8 @@ int mail_index_sync_begin_to(struct mail_index *index,
 /* Returns TRUE if it currently looks like syncing would return changes. */
 bool mail_index_sync_have_any(struct mail_index *index,
 			      enum mail_index_sync_flags flags);
+/* Returns TRUE if it currently looks like syncing would return expunges. */
+bool mail_index_sync_have_any_expunges(struct mail_index *index);
 /* Returns the log file seq+offsets for the area which this sync is handling. */
 void mail_index_sync_get_offsets(struct mail_index_sync_ctx *ctx,
 				 uint32_t *seq1_r, uoff_t *offset1_r,
@@ -381,6 +405,11 @@ void mail_index_sync_set_commit_result(struct mail_index_sync_ctx *ctx,
    MAIL_TRANSACTION_LOG_LOCK_WARN_SECS seconds. Usually this is called because
    the caller itself already logged a warning about it. */
 void mail_index_sync_no_warning(struct mail_index_sync_ctx *ctx);
+/* If a warning is logged because syncing took over
+   MAIL_TRANSACTION_LOG_LOCK_WARN_SECS seconds, log this as the reason for the
+   syncing. */
+void mail_index_sync_set_reason(struct mail_index_sync_ctx *ctx,
+				const char *reason);
 /* Commit synchronization by writing all changes to mail index file. */
 int mail_index_sync_commit(struct mail_index_sync_ctx **ctx);
 /* Rollback synchronization - none of the changes listed by sync_next() are
@@ -494,6 +523,9 @@ void mail_index_update_highest_modseq(struct mail_index_transaction *t,
 /* Reset the index before committing this transaction. This is usually done
    only when UIDVALIDITY changes. */
 void mail_index_reset(struct mail_index_transaction *t);
+/* Remove MAIL_INDEX_HDR_FLAG_FSCKD from header if it exists. This must be
+   called only during syncing so that the mailbox is locked. */
+void mail_index_unset_fscked(struct mail_index_transaction *t);
 /* Mark index deleted. No further changes will be possible after the
    transaction has been committed. */
 void mail_index_set_deleted(struct mail_index_transaction *t);
@@ -559,6 +591,12 @@ uint32_t mail_index_ext_register(struct mail_index *index, const char *name,
 				 uint32_t default_hdr_size,
 				 uint16_t default_record_size,
 				 uint16_t default_record_align);
+/* Change an already registered extension's default sizes. */
+void mail_index_ext_register_resize_defaults(struct mail_index *index,
+					     uint32_t ext_id,
+					     uint32_t default_hdr_size,
+					     uint16_t default_record_size,
+					     uint16_t default_record_align);
 /* Returns TRUE and sets ext_id_r if extension with given name is registered. */
 bool mail_index_ext_lookup(struct mail_index *index, const char *name,
 			   uint32_t *ext_id_r);
