@@ -51,7 +51,7 @@ static void mail_transaction_log_2_unlink_old(struct mail_transaction_log *log)
 		return;
 	}
 
-	if (st.st_mtime + MAIL_TRANSACTION_LOG2_STALE_SECS <= ioloop_time &&
+	if (ioloop_time - st.st_mtime >= (time_t)log->index->log_rotate_log2_stale_secs &&
 	    !log->index->readonly)
 		i_unlink_if_exists(log->filepath2);
 }
@@ -59,6 +59,7 @@ static void mail_transaction_log_2_unlink_old(struct mail_transaction_log *log)
 int mail_transaction_log_open(struct mail_transaction_log *log)
 {
 	struct mail_transaction_log_file *file;
+	const char *reason;
 	int ret;
 
 	i_free(log->filepath);
@@ -79,7 +80,7 @@ int mail_transaction_log_open(struct mail_transaction_log *log)
 		return 0;
 
 	file = mail_transaction_log_file_alloc(log, log->filepath);
-	if ((ret = mail_transaction_log_file_open(file)) <= 0) {
+	if ((ret = mail_transaction_log_file_open(file, &reason)) <= 0) {
 		/* leave the file for _create() */
 		log->open_file = file;
 		return ret;
@@ -214,10 +215,10 @@ void mail_transaction_logs_clean(struct mail_transaction_log *log)
 }
 
 #define LOG_WANT_ROTATE(file) \
-	(((file)->sync_offset > MAIL_TRANSACTION_LOG_ROTATE_MIN_SIZE && \
-	  (time_t)(file)->hdr.create_stamp < \
-	   ioloop_time - MAIL_TRANSACTION_LOG_ROTATE_TIME) || \
-	 ((file)->sync_offset > MAIL_TRANSACTION_LOG_ROTATE_MAX_SIZE))
+	(((file)->sync_offset > (file)->log->index->log_rotate_min_size && \
+	  (file)->hdr.create_stamp < \
+	   ioloop_time - (file)->log->index->log_rotate_min_created_ago_secs) || \
+	 ((file)->sync_offset > (file)->log->index->log_rotate_max_size))
 
 bool mail_transaction_log_want_rotate(struct mail_transaction_log *log)
 {
@@ -283,15 +284,18 @@ int mail_transaction_log_rotate(struct mail_transaction_log *log, bool reset)
 }
 
 static int
-mail_transaction_log_refresh(struct mail_transaction_log *log, bool nfs_flush)
+mail_transaction_log_refresh(struct mail_transaction_log *log, bool nfs_flush,
+			     const char **reason_r)
 {
         struct mail_transaction_log_file *file;
 	struct stat st;
 
 	i_assert(log->head != NULL);
 
-	if (MAIL_TRANSACTION_LOG_FILE_IN_MEMORY(log->head))
+	if (MAIL_TRANSACTION_LOG_FILE_IN_MEMORY(log->head)) {
+		*reason_r = "Log is in memory";
 		return 0;
+	}
 
 	if (nfs_flush && log->nfs_flush)
 		nfs_flush_file_handle_cache(log->filepath);
@@ -300,12 +304,14 @@ mail_transaction_log_refresh(struct mail_transaction_log *log, bool nfs_flush)
 			mail_index_file_set_syscall_error(log->index,
 							  log->filepath,
 							  "stat()");
+			*reason_r = t_strdup_printf("stat(%s) failed: %m", log->filepath);
 			return -1;
 		}
 		/* see if the whole directory got deleted */
 		if (nfs_safe_stat(log->index->dir, &st) < 0 &&
 		    errno == ENOENT) {
 			log->index->index_deleted = TRUE;
+			*reason_r = "Index directory was deleted";
 			return -1;
 		}
 
@@ -313,11 +319,14 @@ mail_transaction_log_refresh(struct mail_transaction_log *log, bool nfs_flush)
 		   someone deleted it manually while the index was open. try to
 		   handle this nicely by creating a new log file. */
 		file = log->head;
-		if (mail_transaction_log_create(log, FALSE) < 0)
+		if (mail_transaction_log_create(log, FALSE) < 0) {
+			*reason_r = "Failed to create log";
 			return -1;
+		}
 		i_assert(file->refcount > 0);
 		file->refcount--;
 		log->index->need_recreate = TRUE;
+		*reason_r = "Log created";
 		return 0;
 	} else if (log->head->st_ino == st.st_ino &&
 		   CMP_DEV_T(log->head->st_dev, st.st_dev)) {
@@ -326,11 +335,12 @@ mail_transaction_log_refresh(struct mail_transaction_log *log, bool nfs_flush)
 		   the existing file has already been unlinked here
 		   (in which case inodes could match but point to
 		   different files) */
+		*reason_r = "Log inode is unchanged";
 		return 0;
 	}
 
 	file = mail_transaction_log_file_alloc(log, log->filepath);
-	if (mail_transaction_log_file_open(file) <= 0) {
+	if (mail_transaction_log_file_open(file, reason_r) <= 0) {
 		mail_transaction_log_file_free(&file);
 		return -1;
 	}
@@ -340,6 +350,7 @@ mail_transaction_log_refresh(struct mail_transaction_log *log, bool nfs_flush)
 	if (--log->head->refcount == 0)
 		mail_transaction_logs_clean(log);
 	mail_transaction_log_set_head(log, file);
+	*reason_r = "Log reopened";
 	return 0;
 }
 
@@ -364,9 +375,11 @@ void mail_transaction_log_set_mailbox_sync_pos(struct mail_transaction_log *log,
 
 int mail_transaction_log_find_file(struct mail_transaction_log *log,
 				   uint32_t file_seq, bool nfs_flush,
-				   struct mail_transaction_log_file **file_r)
+				   struct mail_transaction_log_file **file_r,
+				   const char **reason_r)
 {
 	struct mail_transaction_log_file *file;
+	const char *reason;
 	int ret;
 
 	if (file_seq > log->head->hdr.file_seq) {
@@ -374,25 +387,32 @@ int mail_transaction_log_find_file(struct mail_transaction_log *log,
 		if (log->head->locked) {
 			/* transaction log is locked. there's no way a newer
 			   file exists. */
-			return 0;
-		}
-		if (log->index->open_count == 0) {
-			/* we're opening the index and we just opened the
-			   log file. don't waste time checking if there's a
-			   newer one. */
+			*reason_r = "Log is locked - newer log can't exist";
 			return 0;
 		}
 
-		if (mail_transaction_log_refresh(log, FALSE) < 0)
+		if (mail_transaction_log_refresh(log, FALSE, &reason) < 0) {
+			*reason_r = reason;
 			return -1;
+		}
 		if (file_seq > log->head->hdr.file_seq) {
-			if (!nfs_flush || !log->nfs_flush)
+			if (!nfs_flush || !log->nfs_flush) {
+				*reason_r = t_strdup_printf(
+					"Requested newer log than exists: %s", reason);
 				return 0;
+			}
 			/* try again, this time flush attribute cache */
-			if (mail_transaction_log_refresh(log, TRUE) < 0)
+			if (mail_transaction_log_refresh(log, TRUE, &reason) < 0) {
+				*reason_r = t_strdup_printf(
+					"Log refresh with NFS flush failed: %s", reason);
 				return -1;
-			if (file_seq > log->head->hdr.file_seq)
+			}
+			if (file_seq > log->head->hdr.file_seq) {
+				*reason_r = t_strdup_printf(
+					"Requested newer log than exists - "
+					"still after NFS flush: %s", reason);
 				return 0;
+			}
 		}
 	}
 
@@ -403,19 +423,24 @@ int mail_transaction_log_find_file(struct mail_transaction_log *log,
 		}
 	}
 
-	if (MAIL_INDEX_IS_IN_MEMORY(log->index))
+	if (MAIL_INDEX_IS_IN_MEMORY(log->index)) {
+		*reason_r = "Logs are only in memory";
 		return 0;
+	}
 
 	/* see if we have it in log.2 file */
 	file = mail_transaction_log_file_alloc(log, log->filepath2);
-	if ((ret = mail_transaction_log_file_open(file)) <= 0) {
+	if ((ret = mail_transaction_log_file_open(file, reason_r)) <= 0) {
 		mail_transaction_log_file_free(&file);
 		return ret;
 	}
 
 	/* but is it what we expected? */
-	if (file->hdr.file_seq != file_seq)
+	if (file->hdr.file_seq != file_seq) {
+		*reason_r = t_strdup_printf(".log.2 contains file_seq=%u",
+					    file->hdr.file_seq);
 		return 0;
+	}
 
 	*file_r = file;
 	return 1;
@@ -426,6 +451,7 @@ int mail_transaction_log_lock_head(struct mail_transaction_log *log,
 {
 	struct mail_transaction_log_file *file;
 	time_t lock_wait_started, lock_secs = 0;
+	const char *reason;
 	int ret = 0;
 
 	if (!log->log_2_unlink_checked) {
@@ -454,7 +480,7 @@ int mail_transaction_log_lock_head(struct mail_transaction_log *log,
 			return -1;
 
 		file->refcount++;
-		ret = mail_transaction_log_refresh(log, TRUE);
+		ret = mail_transaction_log_refresh(log, TRUE, &reason);
 		if (--file->refcount == 0) {
 			mail_transaction_log_file_unlock(file, t_strdup_printf(
 				"trying to lock head for %s", lock_reason));
