@@ -323,6 +323,11 @@ const char *sql_result_get_error(struct sql_result *result)
 	return result->v.get_error(result);
 }
 
+enum sql_result_error_type sql_result_get_error_type(struct sql_result *result)
+{
+	return result->error_type;
+}
+
 static void
 sql_result_not_connected_free(struct sql_result *result ATTR_UNUSED)
 {
@@ -345,6 +350,20 @@ struct sql_transaction_context *sql_transaction_begin(struct sql_db *db)
 	return db->v.transaction_begin(db);
 }
 
+struct sql_commit1_wrap_ctx {
+	sql_commit_callback_t *callback;
+	void *context;
+};
+
+static void sql_commit1_wrap(const struct sql_commit_result *result,
+			     void *context)
+{
+	struct sql_commit1_wrap_ctx *ctx = context;
+
+	ctx->callback(result->error, ctx->context);
+	i_free(ctx);
+}
+
 #undef sql_transaction_commit
 void sql_transaction_commit(struct sql_transaction_context **_ctx,
 			    sql_commit_callback_t *callback, void *context)
@@ -352,7 +371,49 @@ void sql_transaction_commit(struct sql_transaction_context **_ctx,
 	struct sql_transaction_context *ctx = *_ctx;
 
 	*_ctx = NULL;
-	ctx->db->v.transaction_commit(ctx, callback, context);
+	if (ctx->db->v.transaction_commit != NULL)
+		ctx->db->v.transaction_commit(ctx, callback, context);
+	else {
+		struct sql_commit1_wrap_ctx *wrap;
+
+		wrap = i_new(struct sql_commit1_wrap_ctx, 1);
+		wrap->callback = callback;
+		wrap->context = context;
+		ctx->db->v.transaction_commit2(ctx, sql_commit1_wrap, wrap);
+	}
+}
+
+struct sql_commit2_wrap_ctx {
+	sql_commit2_callback_t *callback;
+	void *context;
+};
+
+static void sql_commit2_wrap(const char *error, void *context)
+{
+	struct sql_commit2_wrap_ctx *ctx = context;
+	struct sql_commit_result result = { .error = error };
+
+	ctx->callback(&result, ctx->context);
+	i_free(ctx);
+}
+
+#undef sql_transaction_commit2
+void sql_transaction_commit2(struct sql_transaction_context **_ctx,
+			     sql_commit2_callback_t *callback, void *context)
+{
+	struct sql_transaction_context *ctx = *_ctx;
+
+	*_ctx = NULL;
+	if (ctx->db->v.transaction_commit2 != NULL)
+		ctx->db->v.transaction_commit2(ctx, callback, context);
+	else {
+		struct sql_commit2_wrap_ctx *wrap;
+
+		wrap = i_new(struct sql_commit2_wrap_ctx, 1);
+		wrap->callback = callback;
+		wrap->context = context;
+		ctx->db->v.transaction_commit(ctx, sql_commit2_wrap, wrap);
+	}
 }
 
 int sql_transaction_commit_s(struct sql_transaction_context **_ctx,
