@@ -33,6 +33,7 @@ static ssize_t i_stream_ssl_read_real(struct istream_private *stream)
 	struct ssl_istream *sstream = (struct ssl_istream *)stream;
 	struct ssl_iostream *ssl_io = sstream->ssl_io;
 	unsigned char buffer[IO_BLOCK_SIZE];
+	size_t max_buffer_size = i_stream_get_max_buffer_size(&stream->istream);
 	size_t orig_max_buffer_size = stream->max_buffer_size;
 	size_t size;
 	ssize_t ret, total_ret;
@@ -42,9 +43,9 @@ static ssize_t i_stream_ssl_read_real(struct istream_private *stream)
 		return -1;
 	}
 
-	if (stream->pos >= stream->max_buffer_size) {
+	if (stream->pos >= max_buffer_size) {
 		i_stream_compress(stream);
-		if (stream->pos >= stream->max_buffer_size)
+		if (stream->pos >= max_buffer_size)
 			return -2;
 	}
 
@@ -62,9 +63,9 @@ static ssize_t i_stream_ssl_read_real(struct istream_private *stream)
 
 	if (!i_stream_try_alloc(stream, 1, &size))
 		i_unreached();
-	if (stream->pos + size > stream->max_buffer_size) {
-		i_assert(stream->max_buffer_size > stream->pos);
-		size = stream->max_buffer_size - stream->pos;
+	if (stream->pos + size > max_buffer_size) {
+		i_assert(max_buffer_size > stream->pos);
+		size = max_buffer_size - stream->pos;
 	}
 
 	while ((ret = SSL_read(ssl_io->ssl,
