@@ -14,10 +14,15 @@ enum imapc_features {
 	IMAPC_FEATURE_NO_EXAMINE		= 0x40,
 	IMAPC_FEATURE_PROXYAUTH			= 0x80,
 	IMAPC_FEATURE_FETCH_MSN_WORKAROUNDS	= 0x100,
-	IMAPC_FEATURE_FETCH_FIX_BROKEN_MAILS	= 0x200
+	IMAPC_FEATURE_FETCH_FIX_BROKEN_MAILS	= 0x200,
+	IMAPC_FEATURE_MODSEQ			= 0x400
 };
 /* </settings checks> */
 
+/*
+ * NOTE: Any additions here should be reflected in imapc_storage_create's
+ * serialization of settings.
+ */
 struct imapc_settings {
 	const char *imapc_host;
 	in_port_t imapc_port;
@@ -35,6 +40,7 @@ struct imapc_settings {
 	const char *imapc_list_prefix;
 	unsigned int imapc_cmd_timeout;
 	unsigned int imapc_max_idle_time;
+	uoff_t imapc_max_line_length;
 
 	const char *pop3_deleted_flag;
 
