@@ -26,7 +26,7 @@ static void mbox_md5_all_more(struct mbox_md5_context *ctx,
 }
 
 static void mbox_md5_all_finish(struct mbox_md5_context *ctx,
-				unsigned char result[16])
+				unsigned char result[STATIC_ARRAY 16])
 {
 	md5_final(&ctx->hdr_md5_ctx, result);
 	i_free(ctx);
