@@ -264,8 +264,13 @@ static int index_list_get_cached_vsize(struct mailbox *box, uoff_t *vsize_r)
 	ret = mailbox_list_index_status(box->list, view, seq,
 					STATUS_MESSAGES | STATUS_UIDNEXT,
 					&status, NULL, &vsize) ? 1 : 0;
-	if (ret > 0 && (vsize.highest_uid + 1 != status.uidnext ||
-			vsize.message_count != status.messages)) {
+	if (status.messages == 0 && status.uidnext > 0) {
+		/* mailbox is empty. its size has to be zero, regardless of
+		   what the vsize header says. */
+		vsize.vsize = 0;
+		ret = 1;
+	} else if (ret > 0 && (vsize.highest_uid + 1 != status.uidnext ||
+			       vsize.message_count != status.messages)) {
 		/* out of date vsize info */
 		ret = 0;
 	}
@@ -686,6 +691,8 @@ static int index_list_update_mailbox(struct mailbox *box)
 		ilist->updating_status = FALSE;
 	}
 
+	struct mail_index_sync_rec sync_rec;
+	while (mail_index_sync_next(list_sync_ctx, &sync_rec)) ;
 	if (mail_index_sync_commit(&list_sync_ctx) < 0) {
 		mailbox_set_index_error(box);
 		return -1;
