@@ -12,6 +12,12 @@
 
 #define MAILBOX_LIST_INDEX_REFRESH_DELAY_MSECS 1000
 
+/* dovecot.list.index.log doesn't have to be kept for that long. */
+#define MAILBOX_LIST_INDEX_LOG_ROTATE_MIN_SIZE (8*1024)
+#define MAILBOX_LIST_INDEX_LOG_ROTATE_MAX_SIZE (64*1024)
+#define MAILBOX_LIST_INDEX_LOG_ROTATE_SECS_AGO (5*60)
+#define MAILBOX_LIST_INDEX_LOG2_STALE_SECS (10*60)
+
 static void mailbox_list_index_init_finish(struct mailbox_list *list);
 
 struct mailbox_list_index_module mailbox_list_index_module =
@@ -73,6 +79,11 @@ int mailbox_list_index_index_open(struct mailbox_list *list)
 	mail_index_set_permissions(ilist->index, perm.file_create_mode,
 				   perm.file_create_gid,
 				   perm.file_create_gid_origin);
+	mail_index_set_log_rotation(ilist->index,
+				    MAILBOX_LIST_INDEX_LOG_ROTATE_MIN_SIZE,
+				    MAILBOX_LIST_INDEX_LOG_ROTATE_MAX_SIZE,
+				    MAILBOX_LIST_INDEX_LOG_ROTATE_SECS_AGO,
+				    MAILBOX_LIST_INDEX_LOG2_STALE_SECS);
 
 	mail_index_set_fsync_mode(ilist->index, set->parsed_fsync_mode, 0);
 	mail_index_set_lock_method(ilist->index, set->parsed_lock_method,
@@ -233,6 +244,7 @@ mailbox_list_index_generate_name(struct mailbox_list_index *ilist,
 	name = p_strdup_printf(ilist->mailbox_pool, "unknown-%s",
 			       guid_128_to_string(guid));
 	node->name = name;
+	node->flags |= MAILBOX_LIST_INDEX_FLAG_CORRUPTED_NAME;
 
 	hash_table_insert(ilist->mailbox_names,
 			  POINTER_CAST(node->name_id), name);
@@ -254,11 +266,23 @@ mailbox_list_index_node_hash(const struct mailbox_list_index_node *node)
 		POINTER_CAST_TO(node->parent, unsigned int);
 }
 
+static bool node_has_parent(const struct mailbox_list_index_node *parent,
+			    const struct mailbox_list_index_node *node)
+{
+	const struct mailbox_list_index_node *n;
+
+	for (n = parent; n != NULL; n = n->parent) {
+		if (n == node)
+			return TRUE;
+	}
+	return FALSE;
+}
+
 static int mailbox_list_index_parse_records(struct mailbox_list_index *ilist,
 					    struct mail_index_view *view,
 					    const char **error_r)
 {
-	struct mailbox_list_index_node *node;
+	struct mailbox_list_index_node *node, *parent;
 	HASH_TABLE(struct mailbox_list_index_node *,
 		   struct mailbox_list_index_node *) duplicate_hash;
 	const struct mail_index_record *rec;
@@ -292,7 +316,8 @@ static int mailbox_list_index_parse_records(struct mailbox_list_index *ilist,
 		node->name = hash_table_lookup(ilist->mailbox_names,
 					       POINTER_CAST(irec->name_id));
 		if (node->name == NULL) {
-			*error_r = "name_id not in index header";
+			*error_r = t_strdup_printf(
+				"name_id=%u not in index header", irec->name_id);
 			if (ilist->has_backing_store)
 				break;
 			/* generate a new name and use it */
@@ -305,7 +330,7 @@ static int mailbox_list_index_parse_records(struct mailbox_list_index *ilist,
 	/* do a second scan to create the actual mailbox tree hierarchy.
 	   this is needed because the parent_uid may be smaller or higher than
 	   the current node's uid */
-	if (*error_r != NULL)
+	if (*error_r != NULL && ilist->has_backing_store)
 		count = 0;
 	for (seq = 1; seq <= count; seq++) {
 		mail_index_lookup_uid(view, seq, &uid);
@@ -318,37 +343,54 @@ static int mailbox_list_index_parse_records(struct mailbox_list_index *ilist,
 
 		if (irec->parent_uid != 0) {
 			/* node should have a parent */
-			node->parent = mailbox_list_index_lookup_uid(ilist,
-							irec->parent_uid);
-			if (node->parent != NULL) {
-				node->next = node->parent->children;
-				node->parent->children = node;
+			parent = mailbox_list_index_lookup_uid(ilist,
+							       irec->parent_uid);
+			if (parent == NULL) {
+				*error_r = t_strdup_printf(
+					"parent_uid=%u points to nonexistent record",
+					irec->parent_uid);
+				if (ilist->has_backing_store)
+					break;
+				/* just place it under the root */
+				node->corrupted_parent = TRUE;
+			} else if (node_has_parent(parent, node)) {
+				*error_r = t_strdup_printf(
+					"parent_uid=%u loops to node itself (%s)",
+					uid, node->name);
+				if (ilist->has_backing_store)
+					break;
+				/* just place it under the root */
+				node->corrupted_parent = TRUE;
+			} else {
+				node->parent = parent;
+				node->next = parent->children;
+				parent->children = node;
 				continue;
 			}
-			*error_r = "parent_uid points to nonexistent record";
-			if (ilist->has_backing_store)
-				break;
-			/* just place it under the root */
 		}
 		if (hash_table_lookup(duplicate_hash, node) == NULL)
 			hash_table_insert(duplicate_hash, node, node);
 		else {
+			const char *old_name = node->name;
 			guid_128_t guid;
 
 			if (ilist->has_backing_store) {
-				*error_r = "Duplicate mailbox in index";
+				*error_r = t_strdup_printf(
+					"Duplicate mailbox '%s' in index",
+					node->name);
 				break;
 			}
 
 			/* we have only the mailbox list index and this node
 			   may have a different GUID, so rename it. */
 			guid_128_generate(guid);
+			node->flags |= MAILBOX_LIST_INDEX_FLAG_CORRUPTED_NAME;
 			node->name = p_strdup_printf(ilist->mailbox_pool,
 						     "%s-duplicate-%s", node->name,
 						     guid_128_to_string(guid));
 			*error_r = t_strdup_printf(
-				"Duplicate mailbox in index, renaming to %s",
-				node->name);
+				"Duplicate mailbox '%s' in index, renaming to %s",
+				old_name, node->name);
 		}
 		node->next = ilist->mailbox_tree;
 		ilist->mailbox_tree = node;
@@ -371,6 +413,8 @@ int mailbox_list_index_parse(struct mailbox_list *list,
 		/* nothing changed */
 		return 0;
 	}
+	if ((hdr->flags & MAIL_INDEX_HDR_FLAG_FSCKD) != 0)
+		ilist->call_corruption_callback = TRUE;
 
 	mailbox_list_index_reset(ilist);
 	ilist->sync_log_file_seq = hdr->log_file_seq;
@@ -383,6 +427,8 @@ int mailbox_list_index_parse(struct mailbox_list *list,
 			mail_index_mark_corrupted(ilist->index);
 			return -1;
 		}
+		ilist->call_corruption_callback = TRUE;
+		ilist->corrupted_names_or_parents = TRUE;
 	}
 	if (mailbox_list_index_parse_records(ilist, view, &error) < 0) {
 		mailbox_list_set_critical(list,
@@ -394,6 +440,8 @@ int mailbox_list_index_parse(struct mailbox_list *list,
 		}
 		/* FIXME: find any missing mailboxes, add them and write the
 		   index back. */
+		ilist->call_corruption_callback = TRUE;
+		ilist->corrupted_names_or_parents = TRUE;
 	}
 	return 0;
 }
@@ -456,6 +504,9 @@ int mailbox_list_index_refresh_force(struct mailbox_list *list)
 		ret = mailbox_list_index_parse(list, view, FALSE);
 	}
 	mail_index_view_close(&view);
+
+	if (mailbox_list_index_handle_corruption(list) < 0)
+		ret = -1;
 	return ret;
 }
 
@@ -504,6 +555,51 @@ void mailbox_list_index_refresh_later(struct mailbox_list *list)
 	}
 }
 
+int mailbox_list_index_handle_corruption(struct mailbox_list *list)
+{
+	struct mailbox_list_index *ilist = INDEX_LIST_CONTEXT(list);
+	struct mail_storage *const *storagep;
+	int ret = 0;
+
+	if (!ilist->call_corruption_callback)
+		return 0;
+
+	/* make sure we don't recurse */
+	if (ilist->handling_corruption)
+		return 0;
+	ilist->handling_corruption = TRUE;
+
+	array_foreach(&list->ns->all_storages, storagep) {
+		if ((*storagep)->v.list_index_corrupted != NULL) {
+			if ((*storagep)->v.list_index_corrupted(*storagep) < 0)
+				ret = -1;
+			else {
+				/* FIXME: implement a generic handler that
+				   just lists mailbox directories in filesystem
+				   and adds the missing ones to the index. */
+			}
+		}
+	}
+	if (ret == 0)
+		ret = mailbox_list_index_set_uncorrupted(list);
+	ilist->handling_corruption = FALSE;
+	return ret;
+}
+
+int mailbox_list_index_set_uncorrupted(struct mailbox_list *list)
+{
+	struct mailbox_list_index *ilist = INDEX_LIST_CONTEXT(list);
+	struct mailbox_list_index_sync_context *sync_ctx;
+
+	ilist->call_corruption_callback = FALSE;
+
+	if (mailbox_list_index_sync_begin(list, &sync_ctx) < 0)
+		return -1;
+
+	mail_index_unset_fscked(sync_ctx->trans);
+	return mailbox_list_index_sync_end(&sync_ctx, TRUE);
+}
+
 static void mailbox_list_index_deinit(struct mailbox_list *list)
 {
 	struct mailbox_list_index *ilist = INDEX_LIST_CONTEXT(list);
