@@ -43,6 +43,7 @@ enum mailbox_list_index_flags {
 	MAILBOX_LIST_INDEX_FLAG_NONEXISTENT = MAIL_DELETED,
 	MAILBOX_LIST_INDEX_FLAG_NOSELECT = MAIL_DRAFT,
 	MAILBOX_LIST_INDEX_FLAG_NOINFERIORS = MAIL_ANSWERED,
+	MAILBOX_LIST_INDEX_FLAG_CORRUPTED_NAME = MAIL_SEEN,
 
 	/* set during syncing for mailboxes that still exist */
 	MAILBOX_LIST_INDEX_FLAG_SYNC_EXISTS = MAIL_FLAGGED
@@ -80,6 +81,8 @@ struct mailbox_list_index_node {
 
 	uint32_t name_id, uid;
 	enum mailbox_list_index_flags flags;
+	/* parent_uid is corrupted on disk - need to update it */
+	bool corrupted_parent;
 	const char *name;
 };
 
@@ -89,7 +92,7 @@ struct mailbox_list_index {
 	const char *path;
 	struct mail_index *index;
 	uint32_t ext_id, msgs_ext_id, hmodseq_ext_id, subs_hdr_ext_id;
-	uint32_t vsize_ext_id;
+	uint32_t vsize_ext_id, first_saved_ext_id;
 	struct timeval last_refresh_timeval;
 
 	pool_t mailbox_pool;
@@ -113,6 +116,9 @@ struct mailbox_list_index {
 	unsigned int updating_status:1;
 	unsigned int has_backing_store:1;
 	unsigned int index_last_check_changed:1;
+	unsigned int corrupted_names_or_parents:1;
+	unsigned int handling_corruption:1;
+	unsigned int call_corruption_callback:1;
 };
 
 struct mailbox_list_index_iterate_context {
@@ -152,6 +158,8 @@ int mailbox_list_index_refresh(struct mailbox_list *list);
 /* Refresh the index regardless of when the last refresh was done. */
 int mailbox_list_index_refresh_force(struct mailbox_list *list);
 void mailbox_list_index_refresh_later(struct mailbox_list *list);
+int mailbox_list_index_handle_corruption(struct mailbox_list *list);
+int mailbox_list_index_set_uncorrupted(struct mailbox_list *list);
 
 struct mailbox_list_index_node *
 mailbox_list_index_node_find_sibling(struct mailbox_list_index_node *node,
