@@ -105,6 +105,7 @@ void mail_search_arg_init(struct mail_search_args *args,
 				thread_args->pool = args->pool;
 				thread_args->args = arg->value.subargs;
 				thread_args->simplified = TRUE;
+				thread_args->init_refcount = 1;
 				/* simplification should have unnested all
 				   inthreads, so we'll assume that
 				   have_inthreads=FALSE */
@@ -609,7 +610,7 @@ bool mail_search_arg_one_equals(const struct mail_search_arg *arg1,
 	case SEARCH_FLAGS:
 		return arg1->value.flags == arg2->value.flags;
 	case SEARCH_KEYWORDS:
-		return strcasecmp(arg1->value.str, arg2->value.str);
+		return strcasecmp(arg1->value.str, arg2->value.str) == 0;
 
 	case SEARCH_BEFORE:
 	case SEARCH_ON:
