@@ -23,6 +23,8 @@ struct mail_user {
 	/* User's creator if such exists. For example for autocreated shared
 	   mailbox users their creator is the logged in user. */
 	struct mail_user *creator;
+	/* Set if user was created via mail_storage_service. */
+	struct mail_storage_service_user *_service_user;
 
 	const char *username;
 	/* don't access the home directly. It may be set lazily. */
@@ -34,6 +36,10 @@ struct mail_user {
 	const char *session_id;
 	struct ip_addr *local_ip, *remote_ip;
 	const char *auth_token, *auth_user;
+	const char *const *userdb_fields;
+	/* Timestamp when this session was initially created. Most importantly
+	   this stays the same after IMAP client is hibernated and restored. */
+	time_t session_create_time;
 
 	const struct var_expand_table *var_expand_table;
 	/* If non-NULL, fail the user initialization with this error.
@@ -47,7 +53,6 @@ struct mail_user {
 	struct mail_storage *storages;
 	ARRAY(const struct mail_storage_hooks *) hooks;
 
-	struct mountpoint_list *mountpoints;
 	normalizer_func_t *default_normalizer;
 	/* Filled lazily by mailbox_attribute_*() when accessing attributes. */
 	struct dict *_attr_dict;
@@ -67,6 +72,10 @@ struct mail_user {
 	unsigned int autocreated:1;
 	/* mail_user_init() has been called */
 	unsigned int initialized:1;
+	/* SET_STR_VARS in user's all settings have been expanded.
+	   This happens near the beginning of the user initialization,
+	   so this is rarely needed to be checked. */
+	unsigned int settings_expanded:1;
 	/* Shortcut to mail_storage_settings.mail_debug */
 	unsigned int mail_debug:1;
 	/* If INBOX can't be opened, log an error, but only once. */
@@ -81,6 +90,12 @@ struct mail_user {
 	unsigned int deinitializing:1;
 	/* Enable administrator user commands for the user */
 	unsigned int admin:1;
+	/* Enable all statistics gathering */
+	unsigned int stats_enabled:1;
+	/* Enable autoexpunging at deinit. */
+	unsigned int autoexpunge_enabled:1;
+	/* This session was restored (e.g. IMAP unhibernation) */
+	unsigned int session_restored:1;
 };
 
 struct mail_user_module_register {
@@ -93,6 +108,7 @@ union mail_user_module_context {
 };
 extern struct mail_user_module_register mail_user_module_register;
 extern struct auth_master_connection *mail_user_auth_master_conn;
+extern const struct var_expand_func_table *mail_user_var_expand_func_table;
 
 struct mail_user *mail_user_alloc(const char *username,
 				  const struct setting_parser_info *set_info,
@@ -151,12 +167,6 @@ const char *mail_user_home_expand(struct mail_user *user, const char *path);
 int mail_user_try_home_expand(struct mail_user *user, const char **path);
 /* Returns unique user+ip identifier for anvil. */
 const char *mail_user_get_anvil_userip_ident(struct mail_user *user);
-/* Returns FALSE if path is in a mountpoint that should be mounted,
-   but isn't mounted. In such a situation it's better to fail than to attempt
-   any kind of automatic file/dir creations. error_r gives an error about which
-   mountpoint should be mounted. */
-bool mail_user_is_path_mounted(struct mail_user *user, const char *path,
-			       const char **error_r);
 
 /* Basically the same as mail_storage_find_class(), except automatically load
    storage plugins when needed. */
