@@ -46,20 +46,28 @@ struct {
 	{ "ANSWERED NOT FLAGGED SEEN NOT DRAFT", "(ANSWERED SEEN) NOT (FLAGGED) NOT (DRAFT)" },
 	{ "OR NOT ANSWERED NOT SEEN", "NOT (ANSWERED SEEN)" },
 
-	{ "1:5 10:20", "1:5,10:20" },
-	{ "1:5 NOT 10:20", "1:5 NOT 10:20" },
-	{ "1:5 NOT 10:20 NOT 30:40", "1:5 NOT 10:20 NOT 30:40" },
-	{ "OR 1:5 NOT 10:20", "(OR 1:5 NOT 10:20)" },
-	{ "OR 1:5 OR NOT 10:20 NOT 30:40", "(OR 1:5 NOT 10:20,30:40)" },
+	{ "1:* 1:*", "ALL" },
+	{ "OR 1:5 6:*", "ALL" },
 
-	{ "UID 1:5 UID 10:20", "UID 1:5,10:20" },
-	{ "UID 1:5 NOT UID 10:20", "UID 1:5 NOT UID 10:20" },
-	{ "UID 1:5 NOT UID 10:20 NOT UID 30:40", "UID 1:5 NOT UID 10:20 NOT UID 30:40" },
-	{ "OR UID 1:5 NOT UID 10:20", "(OR UID 1:5 NOT UID 10:20)" },
-	{ "OR UID 1:5 OR NOT UID 10:20 NOT UID 30:40", "(OR UID 1:5 NOT UID 10:20,30:40)" },
+	{ "2:* 2:*", "2:4294967295" },
+	{ "OR 2:* 2:*", "2:4294967295" },
+
+	{ "1:5 6:7", "NOT ALL" },
+	{ "1:5 3:7", "3:5" },
+	{ "1:5 3:7 4:9", "4:5" },
+	{ "1:5 OR 3:4 4:6", "3:5" },
+	{ "OR 1 2", "1:2" },
+	{ "NOT 1,3:5", "2,6:4294967295" },
+	{ "NOT 1:100 NOT 50:200", "201:4294967295" },
+	{ "OR NOT 1:100 NOT 50:200", "1:49,101:4294967295" },
+
+	{ "UID 1:5 UID 6:7", "NOT ALL" },
+	{ "UID 1:5 UID 3:7", "UID 3:5" },
+	{ "OR UID 1 UID 2", "UID 1:2" },
+	{ "NOT UID 1,3:5", "UID 2,6:4294967295" },
 
 	{ "1:5 UID 10:20", "1:5 UID 10:20" },
-	{ "1:5 NOT UID 10:20", "1:5 NOT UID 10:20" },
+	{ "1:5 NOT UID 10:20", "1:5 UID 1:9,21:4294967295" },
 
 	{ "BEFORE 03-Aug-2014 BEFORE 01-Aug-2014 BEFORE 02-Aug-2014", "BEFORE \"01-Aug-2014\"" },
 	{ "OR BEFORE 01-Aug-2014 BEFORE 02-Aug-2014", "BEFORE \"02-Aug-2014\"" },
