@@ -7,6 +7,14 @@ void test_istream_set_size(struct istream *input, uoff_t size);
 void test_istream_set_allow_eof(struct istream *input, bool allow);
 void test_istream_set_max_buffer_size(struct istream *input, size_t size);
 
+struct ostream *test_ostream_create(buffer_t *output);
+struct ostream *test_ostream_create_nonblocking(buffer_t *output,
+						size_t max_internal_buffer_size);
+/* When output->used reaches max_size, start buffering output internally.
+   When internal buffer reaches max_internal_buffer_size, start returning 0 for
+   o_stream_send*(). */
+void test_ostream_set_max_output_size(struct ostream *output, size_t max_size);
+
 void test_begin(const char *name);
 #define test_assert(code) STMT_START { \
 	if (!(code)) test_assert_failed(#code, __FILE__, __LINE__); \
@@ -33,6 +41,10 @@ bool test_has_failed(void);
 void test_expect_errors(unsigned int expected);
 void test_expect_error_string(const char *substr); /* expect just 1 message matching the printf format */
 void test_expect_no_more_errors(void);
+/* Note that test_expect_error{s,_string}() effectively begin with a check equivalent
+   to test_expect_no_more_errors(), so you don't need the latter explicitly if following
+   it with either of the former.*/
+
 void test_end(void);
 
 void test_out(const char *name, bool success);
@@ -41,13 +53,35 @@ void test_out_reason(const char *name, bool success, const char *reason)
 	ATTR_NULL(3);
 
 int test_run(void (*test_functions[])(void));
+struct named_test {
+	const char *name;
+	void (*func)(void);
+};
+int test_run_named(struct named_test tests[], const char *match);
+
+#define TEST_DECL(x) void x(void);
+#define TEST_NAMELESS(x) x, /* Were you to want to use the X trick but not name the tests */
+#define TEST_NAMED(x) { .name = #x , .func = x },
 
 enum fatal_test_state {
 	FATAL_TEST_FINISHED, /* no more test stages, don't call again */
 	FATAL_TEST_FAILURE,  /* single stage has failed, continue */
 	FATAL_TEST_ABORT,    /* something's gone horrifically wrong */
 };
+struct named_fatal {
+	const char *name;
+	enum fatal_test_state (*func)(int);
+};
 int test_run_with_fatals(void (*test_functions[])(void),
 			 enum fatal_test_state (*fatal_functions[])(int));
+int test_run_named_with_fatals(const char *match, struct named_test tests[],
+			       struct named_fatal fatals[]);
+
+#define FATAL_DECL(x) enum fatal_test_state x(int);
+#define FATAL_NAMELESS(x) x, /* Were you to want to use the X trick but not name the tests */
+#define FATAL_NAMED(x) { .name = #x , .func = x },
+
+/* If a fork() wants to exit(), then this will avoid valgrind leak errors */
+void test_exit(int status) ATTR_NORETURN;
 
 #endif
