@@ -185,7 +185,8 @@ void buffer_write(buffer_t *_buf, size_t pos,
 	struct real_buffer *buf = (struct real_buffer *)_buf;
 
 	buffer_check_limits(buf, pos, data_size);
-	memcpy(buf->w_buffer + pos, data, data_size);
+	if (data_size > 0)
+		memcpy(buf->w_buffer + pos, data, data_size);
 }
 
 void buffer_append(buffer_t *buf, const void *data, size_t data_size)
@@ -363,3 +364,28 @@ void buffer_verify_pool(buffer_t *_buf)
 		i_assert(ret == buf->w_buffer);
 	}
 }
+
+void buffer_truncate_rshift_bits(buffer_t *buf, size_t bits)
+{
+	/* no-op if it's shorten than bits in any case.. */
+	if (buf->used * 8 < bits) return;
+
+	if (bits > 0) {
+		/* truncate it to closest byte boundary */
+		size_t bytes = ((bits + 7) & -8U)/8;
+		/* remainding bits */
+		bits = bits % 8;
+		buffer_set_used_size(buf, I_MIN(bytes, buf->used));
+		unsigned char *ptr = buffer_get_modifiable_data(buf, &bytes);
+		/* right shift over byte array */
+		if (bits > 0) {
+			for(size_t i=bytes-1;i>0;i--)
+				ptr[i] = (ptr[i]>>(8-bits)) +
+					 ((ptr[i-1]&(0xff>>(bits)))<<bits);
+			ptr[0] = ptr[0]>>(8-bits);
+		}
+	} else {
+		buffer_set_used_size(buf, 0);
+	}
+}
+
