@@ -58,6 +58,11 @@ struct connection_settings {
 
 	bool client;
 	bool dont_send_version;
+	/* By default when only input_args() is used, or when
+	   connection_input_line_default() is used, empty lines aren't allowed
+	   since it would result in additional args[0] == NULL check. Setting
+	   this to TRUE passes it through instead of logging an error. */
+	bool allow_empty_args_input;
 	/* Don't call client_connected() immediately on
 	   connection_client_connect() with UNIX sockets. This is mainly
 	   to make the functionality identical with inet sockets, which may
@@ -80,6 +85,9 @@ struct connection {
 
 	struct timeout *to;
 	time_t last_input;
+	struct timeval last_input_tv;
+	struct timeval connect_started;
+	struct timeval connect_finished;
 
 	/* for IP client: */
 	struct ip_addr ip;
@@ -126,6 +134,9 @@ int connection_verify_version(struct connection *conn, const char *const *args);
 
 /* Returns human-readable reason for why connection was disconnected. */
 const char *connection_disconnect_reason(struct connection *conn);
+/* Returns human-readable reason for why connection timed out,
+   e.g. "No input for 10.023 secs". */
+const char *connection_input_timeout_reason(struct connection *conn);
 
 void connection_switch_ioloop(struct connection *conn);
 
